# -*- coding: utf-8 -*- 
'''
# Copyright (c) 2015 Microsoft Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#  This file was generated and any changes will be overwritten.
'''

from ..request_base import RequestBase
from ..request_builder_base import RequestBuilderBase
from ..model.thumbnail import Thumbnail
import json
import asyncio


class ThumbnailContentRequest(RequestBase):
    def __init__(self, request_url, client, options):
        """Initialize the ThumbnailContentRequest

        Args:
            request_url (str): The url to perform the ThumbnailContentRequest
                on
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request
        """
        super(ThumbnailContentRequest, self).__init__(request_url, client, options)

    def download(self, content_local_path):
        """Downloads the specified Thumbnail.
        
        Args:
            content_local_path (str):
                The path where the Thumbnail should be downloaded to
        """
        self.download_item(content_local_path)

    @asyncio.coroutine
    def download_async(self, content_local_path):
        """
        Downloads the specified Thumbnail in async.
        
        Args:
            content_local_path (str):
                The path where the Thumbnail should be downloaded to
        """
        future = self._client._loop.run_in_executor(None,
                                                    self.download,
                                                    content_local_path)
        yield from future

class ThumbnailContentRequestBuilder(RequestBuilderBase):

    def __init__(self, request_url, client):
        """Initialize the ThumbnailContentRequestBuilder
        
        Args:
            request_url (str): The request URL to initialize
                the ThumbnailContentRequestBuilder at
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client to use for requests made by the
                ThumbnailContentRequestBuilder
        """
        super(ThumbnailContentRequestBuilder, self).__init__(request_url, client)

    def request(self):
        """Builds the ThumbnailContentRequest

        Returns:
            :class:`ThumbnailContentRequest<onedrivesdk.request.thumbnail_content.ThumbnailContentRequest>`:
                The ThumbnailContentRequest
        """
        return ThumbnailContentRequest(self._request_url, self._client, None)


# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.auto_tagged import AutoTagged
from ..one_drive_object_base import OneDriveObjectBase


class Tag(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def auto_tagged(self):
        """
        Gets and sets the autoTagged
        
        Returns: 
            :class:`AutoTagged<onedrivesdk.model.auto_tagged.AutoTagged>`:
                The autoTagged
        """
        if "autoTagged" in self._prop_dict:
            if isinstance(self._prop_dict["autoTagged"], OneDriveObjectBase):
                return self._prop_dict["autoTagged"]
            else :
                self._prop_dict["autoTagged"] = AutoTagged(self._prop_dict["autoTagged"])
                return self._prop_dict["autoTagged"]

        return None

    @auto_tagged.setter
    def auto_tagged(self, val):
        self._prop_dict["autoTagged"] = val

    @property
    def name(self):
        """
        Gets and sets the name
        
        Returns:
            str:
                The name
        """
        if "name" in self._prop_dict:
            return self._prop_dict["name"]
        else:
            return None

    @name.setter
    def name(self, val):
        self._prop_dict["name"] = val




'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''


class Option(object):

    def __init__(self, key, value):
        """Initialize the Option, used for passing
        options into requests

        Args:
            key (str): The key for the option
            value (str): The value for the option

        """
        self._key = key
        self._value = value

    @property
    def key(self):
        """Gets and sets the key of the :class:`Option`

        Returns:
            str: The key"""
        return self._key

    @key.setter
    def key(self, value):
        self._key = value

    @property
    def value(self):
        """Gets and sets the value of the :class:`Option`

        Returns:
            str: The value"""
        return self._value

    @value.setter
    def value(self, value):
        self._value = value


class QueryOption(Option):
    """Option used for the query string of a request"""
    pass


class HeaderOption(Option):
    """Option used for the header of the request"""
    pass



'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''
from __future__ import unicode_literals


class OneDriveError(Exception):

    def __init__(self, prop_dict, status_code):
        """Initialize a OneDriveError given the JSON
        error response dictionary, and the HTTP status code

        Args:
            prop_dict (dict): A dictionary containing the response
                from OneDrive
            status_code (int): The HTTP status code (ex. 200, 201, etc.)
        """
        if "code" not in prop_dict or "message" not in prop_dict:
            prop_dict["code"] = ErrorCode.Malformed
            prop_dict["message"] = "The received response was malformed"
            super(OneDriveError, self).__init__(prop_dict["code"]+" - "+prop_dict["message"])
        else:
            super(OneDriveError, self).__init__(prop_dict["code"]+" - "+prop_dict["message"])
        self._prop_dict = prop_dict
        self._status_code = status_code

    @property
    def status_code(self):
        """The HTTP status code

        Returns:
            int: The HTTP status code
        """
        return self._status_code

    @property
    def code(self):
        """The OneDrive error code sent back in
        the response. Possible codes can be found
        in the :class:`ErrorCode` enum.

        Returns:
            str: The error code
        """
        return self._prop_dict["code"]

    @property
    def inner_error(self):
        """Creates a OneDriveError object from the specified inner 
        error within the response.

        Returns:
            :class:`OneDriveError`: Error from within the inner
                response
        """
        return OneDriveError(self._prop_dict["innererror"], self.status_code) if "innererror" in self._prop_dict else None

    def matches(self, code):
        """Recursively searches the :class:`OneDriveError` to find
        if the specified code was found

        Args:
            code (str): The error code to search for

        Returns:
            bool: True if the error code was found, false otherwise
        """
        if self.code == code:
            return True

        return False if self.inner_error is None else self.inner_error.matches(code)


class ErrorCode(object):
    #: Access was denied to the resource
    AccessDenied = "accessDenied"
    #: The activity limit has been reached
    ActivityLimitReached = "activityLimitReached"
    #: A general exception occured
    GeneralException = "generalException"
    #: An invalid range was provided
    InvalidRange = "invalidRange"
    #: An invalid request was provided
    InvalidRequest = "invalidRequest"
    #: The requested resource was not found
    ItemNotFound = "itemNotFound"
    #: Malware was detected in the resource
    MalwareDetected = "malwareDetected"
    #: The name already exists
    NameAlreadyExists = "nameAlreadyExists"
    #: The action was not allowed
    NotAllowed = "notAllowed"
    #: The action was not supported
    NotSupported = "notSupported"
    #: The resource was modified
    ResourceModified = "resourceModified"
    #: A resync is required
    ResyncRequired = "resyncRequired"
    #: The OneDrive service is not available
    ServiceNotAvailable = "serviceNotAvailable"
    #: The quota for this OneDrive has been reached
    QuotaLimitReached = "quotaLimitReached"
    #: The user is unauthenticated
    Unauthenticated = "unauthenticated"

    #: The response was malformed
    Malformed = "malformed"



# -*- coding: utf-8 -*- 
'''
# Copyright (c) 2015 Microsoft Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#  This file was generated and any changes will be overwritten.
'''

from ..model.permission import Permission
from ..request_base import RequestBase
from ..request_builder_base import RequestBuilderBase
from ..options import *
import json


class ItemCreateLinkRequest(RequestBase):

    def __init__(self, request_url, client, options, type):
        super(ItemCreateLinkRequest, self).__init__(request_url, client, options)
        self.method = "POST"
        self.body_options={}

        if type:
            self.body_options["type"] = type

    @property
    def body_options(self):
        return self._body_options

    @body_options.setter
    def body_options(self, value):
        self._body_options=value

    def post(self):
        """Sends the POST request
        
        Returns: 
            :class:`Permission<onedrivesdk.model.permission.Permission>`:
                The resulting entity from the operation
        """
        self.content_type = "application/json"
        entity = Permission(json.loads(self.send(self.body_options).content))
        return entity



class ItemCreateLinkRequestBuilder(RequestBuilderBase):

    def __init__(self, request_url, client, type):
        super(ItemCreateLinkRequestBuilder, self).__init__(request_url, client)
        self._method_options = {}

        self._method_options["type"] = type

    def request(self, options=None):
        """Builds the request for the ItemCreateLink
        
        Args:
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                Default to None, list of options to include in the request

        Returns: 
            :class:`ItemCreateLinkRequest<onedrivesdk.request.item_create_link.ItemCreateLinkRequest>`:
                The request
        """
        req = ItemCreateLinkRequest(self._request_url, self._client, options, self._method_options["type"])
        return req

    def post(self):
        """Sends the POST request
        
        Returns:
            :class:`Permission<onedrivesdk.model.permission.Permission>`:
            The resulting Permission from the operation
        """
        return self.request().post()




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.identity_set import IdentitySet
from datetime import datetime
from ..one_drive_object_base import OneDriveObjectBase


class Subscription(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def app_type(self):
        """
        Gets and sets the appType
        
        Returns:
            str:
                The appType
        """
        if "appType" in self._prop_dict:
            return self._prop_dict["appType"]
        else:
            return None

    @app_type.setter
    def app_type(self, val):
        self._prop_dict["appType"] = val

    @property
    def client_state(self):
        """
        Gets and sets the clientState
        
        Returns:
            str:
                The clientState
        """
        if "clientState" in self._prop_dict:
            return self._prop_dict["clientState"]
        else:
            return None

    @client_state.setter
    def client_state(self, val):
        self._prop_dict["clientState"] = val

    @property
    def created_by(self):
        """
        Gets and sets the createdBy
        
        Returns: 
            :class:`IdentitySet<onedrivesdk.model.identity_set.IdentitySet>`:
                The createdBy
        """
        if "createdBy" in self._prop_dict:
            if isinstance(self._prop_dict["createdBy"], OneDriveObjectBase):
                return self._prop_dict["createdBy"]
            else :
                self._prop_dict["createdBy"] = IdentitySet(self._prop_dict["createdBy"])
                return self._prop_dict["createdBy"]

        return None

    @created_by.setter
    def created_by(self, val):
        self._prop_dict["createdBy"] = val

    @property
    def expiration_date_time(self):
        """
        Gets and sets the expirationDateTime
        
        Returns:
            datetime:
                The expirationDateTime
        """
        if "expirationDateTime" in self._prop_dict:
            if '.' in self._prop_dict["expirationDateTime"]:
                return datetime.strptime(self._prop_dict["expirationDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S.%f")
            else:
                return datetime.strptime(self._prop_dict["expirationDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S")
        else:
            return None

    @expiration_date_time.setter
    def expiration_date_time(self, val):
        self._prop_dict["expirationDateTime"] = val.isoformat()+((".0" if val.time().microsecond == 0 else "")+"Z")

    @property
    def id(self):
        """
        Gets and sets the id
        
        Returns:
            str:
                The id
        """
        if "id" in self._prop_dict:
            return self._prop_dict["id"]
        else:
            return None

    @id.setter
    def id(self, val):
        self._prop_dict["id"] = val

    @property
    def muted(self):
        """
        Gets and sets the muted
        
        Returns:
            bool:
                The muted
        """
        if "muted" in self._prop_dict:
            return self._prop_dict["muted"]
        else:
            return None

    @muted.setter
    def muted(self, val):
        self._prop_dict["muted"] = val

    @property
    def notification_url(self):
        """
        Gets and sets the notificationUrl
        
        Returns:
            str:
                The notificationUrl
        """
        if "notificationUrl" in self._prop_dict:
            return self._prop_dict["notificationUrl"]
        else:
            return None

    @notification_url.setter
    def notification_url(self, val):
        self._prop_dict["notificationUrl"] = val

    @property
    def resource(self):
        """
        Gets and sets the resource
        
        Returns:
            str:
                The resource
        """
        if "resource" in self._prop_dict:
            return self._prop_dict["resource"]
        else:
            return None

    @resource.setter
    def resource(self, val):
        self._prop_dict["resource"] = val

    @property
    def scenarios(self):
        """
        Gets and sets the scenarios
        
        Returns:
            str:
                The scenarios
        """
        if "scenarios" in self._prop_dict:
            return self._prop_dict["scenarios"]
        else:
            return None

    @scenarios.setter
    def scenarios(self, val):
        self._prop_dict["scenarios"] = val




from __future__ import with_statement
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'version.txt'), encoding='utf-8') as f:
    __version__ = f.read()



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.hashes import Hashes
from ..one_drive_object_base import OneDriveObjectBase


class File(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def hashes(self):
        """
        Gets and sets the hashes
        
        Returns: 
            :class:`Hashes<onedrivesdk.model.hashes.Hashes>`:
                The hashes
        """
        if "hashes" in self._prop_dict:
            if isinstance(self._prop_dict["hashes"], OneDriveObjectBase):
                return self._prop_dict["hashes"]
            else :
                self._prop_dict["hashes"] = Hashes(self._prop_dict["hashes"])
                return self._prop_dict["hashes"]

        return None

    @hashes.setter
    def hashes(self, val):
        self._prop_dict["hashes"] = val
    @property
    def mime_type(self):
        """Gets and sets the mimeType
        
        Returns: 
            str:
                The mimeType
        """
        if "mimeType" in self._prop_dict:
            return self._prop_dict["mimeType"]
        else:
            return None

    @mime_type.setter
    def mime_type(self, val):
        self._prop_dict["mimeType"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) 2015 Microsoft Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..collection_base import CollectionRequestBase, CollectionResponseBase, CollectionPageBase
from ..request_builder_base import RequestBuilderBase
import json

class EffectiveRolesCollectionRequest(CollectionRequestBase):

    def __init__(self, request_url, client, options):
        """Initialize the EffectiveRolesCollectionRequest
        
        Args:
            request_url (str): The url to perform the EffectiveRolesCollectionRequest
                on
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request
        """
        super(EffectiveRolesCollectionRequest, self).__init__(request_url, client, options)

    def get(self):
        """Gets the EffectiveRolesCollectionPage

        Returns: 
            :class:`EffectiveRolesCollectionPage<onedrivesdk.request.effective_roles_collection.EffectiveRolesCollectionPage>`:
                The EffectiveRolesCollectionPage
        """
        self.method = "GET"
        collection_response = EffectiveRolesCollectionResponse(json.loads(self.send().content))
        return self._page_from_response(collection_response)


class EffectiveRolesCollectionRequestBuilder(RequestBuilderBase):

    def __getitem__(self, key):
        """Get the strRequestBuilder with the specified key
        
        Args:
            key (str): The key to get a strRequestBuilder for
        
        Returns: 
            :class:`strRequestBuilder<onedrivesdk.request.str_request_builder.strRequestBuilder>`:
                A strRequestBuilder for that key
        """
        return strRequestBuilder(self.append_to_request_url(str(key)), self._client)

    def request(self, expand=None, select=None, top=None, order_by=None, options=None):
        """Builds the EffectiveRolesCollectionRequest
        
        Args:
            expand (str): Default None, comma-seperated list of relationships
                to expand in the response.
            select (str): Default None, comma-seperated list of properties to
                include in the response.
            top (int): Default None, the number of items to return in a result.
            order_by (str): Default None, comma-seperated list of properties
                that are used to sort the order of items in the response.
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request. Defaults to None.

        Returns:
            :class:`EffectiveRolesCollectionRequest<onedrivesdk.request.effective_roles_collection.EffectiveRolesCollectionRequest>`:
                The EffectiveRolesCollectionRequest
        """
        req = EffectiveRolesCollectionRequest(self._request_url, self._client, options)
        req._set_query_options(expand=expand, select=select, top=top, order_by=order_by)
        return req

    def get(self):
        """Gets the EffectiveRolesCollectionPage

        Returns: 
            :class:`EffectiveRolesCollectionPage<onedrivesdk.request.effective_roles_collection.EffectiveRolesCollectionPage>`:
                The EffectiveRolesCollectionPage
        """
        return self.request().get()



class EffectiveRolesCollectionResponse(CollectionResponseBase):

    @property
    def collection_page(self):
        """The collection page stored in the response JSON
        
        Returns:
            :class:`EffectiveRolesCollectionPage<onedrivesdk.request.effective_roles_collection.EffectiveRolesCollectionPage>`:
                The collection page
        """
        if self._collection_page:
            self._collection_page._prop_list = self._prop_dict["value"]
        else:
            self._collection_page = EffectiveRolesCollectionPage(self._prop_dict["value"])

        return self._collection_page


class EffectiveRolesCollectionPage(CollectionPageBase):

    def __getitem__(self, index):
        """Get the str at the index specified
        
        Args:
            index (int): The index of the item to get from the EffectiveRolesCollectionPage

        Returns:
            :class:`str<onedrivesdk.model.str.str>`:
                The str at the index
        """
        return str(self._prop_list[index])

    def effective_roles(self):
        """Get a generator of str within the EffectiveRolesCollectionPage
        
        Yields:
            :class:`str<onedrivesdk.model.str.str>`:
                The next str in the collection
        """
        for item in self._prop_list:
            yield str(item)

    def _init_next_page_request(self, next_page_link, client, options):
        """Initialize the next page request for the EffectiveRolesCollectionPage
        
        Args:
            next_page_link (str): The URL for the next page request
                to be sent to
            client (:class:`OneDriveClient<onedrivesdk.model.one_drive_client.OneDriveClient>`:
                The client to be used for the request
            options (list of :class:`Option<onedrivesdk.options.Option>`:
                A list of options
        """
        self._next_page_request = EffectiveRolesCollectionRequest(next_page_link, client, options)


from ..request.str_request_builder import strRequestBuilder



# -*- coding: utf-8 -*- 
'''
# Copyright (c) 2015 Microsoft Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.items_collection_page import ItemsCollectionPage

class ItemDeltaCollectionPage(ItemsCollectionPage):

    def __init__(self, prop_list, token=None, delta_link=None, next_page_link=None):
        super(ItemDeltaCollectionPage, self).__init__(prop_list)
        self._next_page_link = next_page_link
        self._token = token
        self._delta_link = delta_link

    @property
    def token(self):
        """Gets the token property from the
        ItemDeltaCollectionPage

        Returns:
            str:
                The token property from the ItemDeltaCollectionPage
        """
        return self._token

    @property
    def delta_link(self):
        """Gets the deltaLink property from the
        ItemDeltaCollectionPage

        Returns:
            str:
                The deltaLink property from the ItemDeltaCollectionPage
        """
        return self._delta_link

    @property
    def next_page_link(self):
        """Gets the nextLink property from the
        ItemDeltaCollectionPage

        Returns:
            str:
                The nextLink property from the ItemDeltaCollectionPage
        """
        return self._delta_link


import requests
import json


class ResourceDiscoveryRequest(object):
    def __init__(self):
        self._method = 'GET'
        self._discovery_service_url = 'https://api.office.com/discovery/v2.0/me/services'

    def get_service_info(self, access_token):
        """Send request to discovery service. Return ServiceInfo
        for valid services.

        Args:
            access_token (str): A valid access token for resource
                'https://api.office.com/discovery/'

        Returns:
            List of :class:`ServiceInfo<onedrivesdk.helpers.resource_discovery.ServiceInfo>`:
                ServiceInfo for each service that the caller can access. NOTE: values that
                do not provide access to OneDrive for Business will be excluded (i.e. must have
                capability = 'MyFiles' and service_api_version = 'v2.0'
        """
        headers = {'Authorization': 'Bearer ' + access_token}
        response = json.loads(requests.get(self._discovery_service_url, headers=headers).text)
        service_info_list = [ServiceInfo(x) for x in response['value']]
        trimmed_service_info_list = [si for si in service_info_list
                                     if si.capability == 'MyFiles' and si.service_api_version == 'v2.0']
        return trimmed_service_info_list


class ServiceInfo(object):
    """
    Objects representing ServiceInfo returned by the Discovery Service.
        More info can be found here:
        https://msdn.microsoft.com/en-us/office/office365/api/discovery-service-rest-operations
    """
    def __init__(self, prop_dict={}):
        self._prop_dict = prop_dict

    def __str__(self):
        return 'serviceResourceId: {}\nserviceEndpointUri: {}'\
            .format(self.service_resource_id, self.service_endpoint_uri)

    def _prop_dict_get(self, prop):
        if prop in self._prop_dict:
            return self._prop_dict[prop]
        else:
            return None

    def _prop_dict_set(self, prop, val):
        self._prop_dict[prop] = val

    @property
    def capability(self):
        """

        Returns:
            str: The Capability of the service

        """
        return self._prop_dict_get('capability')

    @capability.setter
    def capability(self, value):
        self._prop_dict_set('capability', value)

    @property
    def service_id(self):
        """

        Returns:
            str: The ServiceId

        """
        return self._prop_dict_get('serviceId')

    @service_id.setter
    def service_id(self, value):
        self._prop_dict_set('serviceId', value)

    @property
    def service_name(self):
        """

        Returns:
            str: The name of the service

        """
        return self._prop_dict_get('serviceName')

    @service_name.setter
    def service_name(self, value):
        self._prop_dict_set('serviceName', value)

    @property
    def service_endpoint_uri(self):
        """

        Returns:
            str: The serviceEndpointUri
                Ex: https://contoso-my.sharepoint.com/personal/alexd_contoso_com

        """
        return self._prop_dict_get('serviceEndpointUri')

    @service_endpoint_uri.setter
    def service_endpoint_uri(self, value):
        self._prop_dict_set('serviceEndpointUri', value)

    @property
    def service_resource_id(self):
        """

        Returns:
            str: the serviceResourceId
                Ex: https://contoso-my.sharepoint.com/

        """
        return self._prop_dict_get('serviceResourceId')

    @service_resource_id.setter
    def service_resource_id(self, value):
        self._prop_dict_set('serviceResourceId', value)

    @property
    def service_api_version(self):
        """

        Returns:
            str: the serviceApiVersion

        """
        return self._prop_dict_get('serviceApiVersion')

    @service_api_version.setter
    def service_api_version(self, value):
        self._prop_dict_set('serviceApiVersion', value)



# -*- coding: utf-8 -*-
'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''

import io
import os

class FileSlice(io.RawIOBase):
    '''
    This class represents a window over a file handle. It will allow only access
    and read of bytes above start and no further than end/length bytes
    '''
    def __init__(self, handle, start, end=None, length=None):
        '''
        Creates new instance of FileSlice on file-like object handle. It behaves
        like normal file object, but only allow reading of bytes above start and
        no further than end/lenght byte

        Args:
            handle (file-like object): file handle object to create a view over.
                File should be open in binary mode
            start (int): start byte number, makring the first byte that can be
                read from FileSlice
            end (int): Optional. Last byte of the file that can be read.
            length (int): Optional. Number of the bytes, starting from the
                start, that can be read from FileSlice

            One of end or length must be provided
        '''
        assert end or length, "You need to provide one of end or length parameter"
        assert not (end and length), "You need to proivde only one parameter: end or length, not both"
        if start < 0:
            raise ValueError("Start of the file smaller than 0")
        if end and end < start:
            raise ValueError("End of the tile smaller than start")
        if length and length < 0:
            raise ValueError("Length smaller than 0")

        self._handle = handle
        self._start = start
        if end:
            self._end = end
        else:
            self._end = start + length
        self._end = min(self._end, os.fstat(handle.fileno()).st_size)
        self.seek(0)

    @property
    def _bytes_left(self):
        current_pos = self._handle.tell()
        return self._end - current_pos

    def close(self):
        # do nothing, someone else might want to process this file
        return

    @property
    def closed(self):
        return self._handle.closed

    def fileno(self):
        return self._handle.fileno()

    def flush(self):
        return self._handle.flush()

    def len(self):
        # this is provided for requests, so it will properly recognize the size of the file
        return self._bytes_left

    def __len__(self):
        # this is provided for requests, so it will properly recognize the size of the file
        return self.len()

    def isatty(self):
        return self._handle.isatty()

    def readable(self):
        return self._handle.readable()

    def read(self, size=-1):
        if size == -1:
            read_size = self._bytes_left
        else:
            read_size = min(size, self._bytes_left)
        return self._handle.read(read_size)

    def readall(self):
        return self._handle.read(self._bytes_left)

    def readinto(self, b):
        if len(b) > self._bytes_left:
            r = self._handle.read(self._bytes_left)
            b[:len(r)] = r
            return len(r)
        return self._handle.readinto(b)

    def readline(self, size=-1):
        return self._handle.readline(max(size, self._bytes_left))

    def readlines(self, hint=-1):
        return self._handle.readlines(max(hint, self._bytes_left))

    def seek(self, offset, whence=io.SEEK_SET):
        if whence == io.SEEK_SET:
            desired_pos = self._start + offset
        if whence == io.SEEK_CUR:
            desired_pos = self._handle.tell() + offset
        if whence == io.SEEK_END:
            desired_pos = self._end + offset

        if desired_pos < self._start:
            raise ValueError("Seeking before the file slice")
        if desired_pos > self._end:
            raise ValueError("Seekeing past the end of file slice")

        ret = self._handle.seek(desired_pos, io.SEEK_SET)
        if ret:
            return ret - self._start
        else:
            return ret

    def seekable(self):
        return self._handle.seekable()

    def tell(self):
        return self._handle.tell() - self._start

    def truncate(self, size=None):
        raise IOError("Operation not supported")

    def writable(self):
        return False

    def write(self, b):
        raise IOError("Operation not supported")

    def writelines(self, lines):
        raise IOError("Operation not supported")




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..one_drive_object_base import OneDriveObjectBase


class ChunkedUploadSessionDescriptor(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def name(self):
        """Gets and sets the name
        
        Returns: 
            str:
                The name
        """
        if "name" in self._prop_dict:
            return self._prop_dict["name"]
        else:
            return None

    @name.setter
    def name(self, val):
        self._prop_dict["name"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from datetime import datetime
from ..one_drive_object_base import OneDriveObjectBase


class Status(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def state(self):
        """Gets and sets the state
        
        Returns: 
            str:
                The state
        """
        if "state" in self._prop_dict:
            return self._prop_dict["state"]
        else:
            return None

    @state.setter
    def state(self, val):
        self._prop_dict["state"] = val

    @property
    def lockdown_date_time(self):
        """Gets and sets the lockdownDateTime
        
        Returns: 
            datetime:
                The lockdownDateTime
        """
        if "lockdownDateTime" in self._prop_dict:
            if '.' in self._prop_dict["lockdownDateTime"]:
                return datetime.strptime(self._prop_dict["lockdownDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S.%f")
            else:
                return datetime.strptime(self._prop_dict["lockdownDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S")
        else:
            return None

    @lockdown_date_time.setter
    def lockdown_date_time(self, val):
        self._prop_dict["lockdownDateTime"] = val.isoformat()+((".0" if val.time().microsecond == 0 else "")+"Z")

    @property
    def lockdown_reasons(self):
        """Gets and sets the lockdownReasons
        
        Returns: 
            str:
                The lockdownReasons
        """
        if "lockdownReasons" in self._prop_dict:
            return self._prop_dict["lockdownReasons"]
        else:
            return None

    @lockdown_reasons.setter
    def lockdown_reasons(self, val):
        self._prop_dict["lockdownReasons"] = val

    @property
    def drive_deletion_date_time(self):
        """Gets and sets the driveDeletionDateTime
        
        Returns: 
            datetime:
                The driveDeletionDateTime
        """
        if "driveDeletionDateTime" in self._prop_dict:
            if '.' in self._prop_dict["driveDeletionDateTime"]:
                return datetime.strptime(self._prop_dict["driveDeletionDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S.%f")
            else:
                return datetime.strptime(self._prop_dict["driveDeletionDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S")
        else:
            return None

    @drive_deletion_date_time.setter
    def drive_deletion_date_time(self, val):
        self._prop_dict["driveDeletionDateTime"] = val.isoformat()+((".0" if val.time().microsecond == 0 else "")+"Z")

    @property
    def last_unlock_date_time(self):
        """Gets and sets the lastUnlockDateTime
        
        Returns: 
            datetime:
                The lastUnlockDateTime
        """
        if "lastUnlockDateTime" in self._prop_dict:
            if '.' in self._prop_dict["lastUnlockDateTime"]:
                return datetime.strptime(self._prop_dict["lastUnlockDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S.%f")
            else:
                return datetime.strptime(self._prop_dict["lastUnlockDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S")
        else:
            return None

    @last_unlock_date_time.setter
    def last_unlock_date_time(self, val):
        self._prop_dict["lastUnlockDateTime"] = val.isoformat()+((".0" if val.time().microsecond == 0 else "")+"Z")

    @property
    def user_unlocks(self):
        """Gets and sets the userUnlocks
        
        Returns: 
            int:
                The userUnlocks
        """
        if "userUnlocks" in self._prop_dict:
            return self._prop_dict["userUnlocks"]
        else:
            return None

    @user_unlocks.setter
    def user_unlocks(self, val):
        self._prop_dict["userUnlocks"] = val

    @property
    def user_unlocks_remaining(self):
        """Gets and sets the userUnlocksRemaining
        
        Returns: 
            int:
                The userUnlocksRemaining
        """
        if "userUnlocksRemaining" in self._prop_dict:
            return self._prop_dict["userUnlocksRemaining"]
        else:
            return None

    @user_unlocks_remaining.setter
    def user_unlocks_remaining(self, val):
        self._prop_dict["userUnlocksRemaining"] = val

    @property
    def support_agent_unlocks(self):
        """Gets and sets the supportAgentUnlocks
        
        Returns: 
            int:
                The supportAgentUnlocks
        """
        if "supportAgentUnlocks" in self._prop_dict:
            return self._prop_dict["supportAgentUnlocks"]
        else:
            return None

    @support_agent_unlocks.setter
    def support_agent_unlocks(self, val):
        self._prop_dict["supportAgentUnlocks"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.thumbnail_set import ThumbnailSet
from ..one_drive_object_base import OneDriveObjectBase


class Identity(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def display_name(self):
        """Gets and sets the displayName
        
        Returns: 
            str:
                The displayName
        """
        if "displayName" in self._prop_dict:
            return self._prop_dict["displayName"]
        else:
            return None

    @display_name.setter
    def display_name(self, val):
        self._prop_dict["displayName"] = val

    @property
    def id(self):
        """Gets and sets the id
        
        Returns: 
            str:
                The id
        """
        if "id" in self._prop_dict:
            return self._prop_dict["id"]
        else:
            return None

    @id.setter
    def id(self, val):
        self._prop_dict["id"] = val

    @property
    def thumbnails(self):
        """
        Gets and sets the thumbnails
        
        Returns: 
            :class:`ThumbnailSet<onedrivesdk.model.thumbnail_set.ThumbnailSet>`:
                The thumbnails
        """
        if "thumbnails" in self._prop_dict:
            if isinstance(self._prop_dict["thumbnails"], OneDriveObjectBase):
                return self._prop_dict["thumbnails"]
            else :
                self._prop_dict["thumbnails"] = ThumbnailSet(self._prop_dict["thumbnails"])
                return self._prop_dict["thumbnails"]

        return None

    @thumbnails.setter
    def thumbnails(self, val):
        self._prop_dict["thumbnails"] = val



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.identity import Identity
from ..one_drive_object_base import OneDriveObjectBase


class OpenWithApp(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def app(self):
        """
        Gets and sets the app
        
        Returns: 
            :class:`Identity<onedrivesdk.model.identity.Identity>`:
                The app
        """
        if "app" in self._prop_dict:
            if isinstance(self._prop_dict["app"], OneDriveObjectBase):
                return self._prop_dict["app"]
            else :
                self._prop_dict["app"] = Identity(self._prop_dict["app"])
                return self._prop_dict["app"]

        return None

    @app.setter
    def app(self, val):
        self._prop_dict["app"] = val
    @property
    def view_url(self):
        """Gets and sets the viewUrl
        
        Returns: 
            str:
                The viewUrl
        """
        if "viewUrl" in self._prop_dict:
            return self._prop_dict["viewUrl"]
        else:
            return None

    @view_url.setter
    def view_url(self, val):
        self._prop_dict["viewUrl"] = val

    @property
    def edit_url(self):
        """Gets and sets the editUrl
        
        Returns: 
            str:
                The editUrl
        """
        if "editUrl" in self._prop_dict:
            return self._prop_dict["editUrl"]
        else:
            return None

    @edit_url.setter
    def edit_url(self, val):
        self._prop_dict["editUrl"] = val

    @property
    def view_post_parameters(self):
        """Gets and sets the viewPostParameters
        
        Returns: 
            str:
                The viewPostParameters
        """
        if "viewPostParameters" in self._prop_dict:
            return self._prop_dict["viewPostParameters"]
        else:
            return None

    @view_post_parameters.setter
    def view_post_parameters(self, val):
        self._prop_dict["viewPostParameters"] = val

    @property
    def edit_post_parameters(self):
        """Gets and sets the editPostParameters
        
        Returns: 
            str:
                The editPostParameters
        """
        if "editPostParameters" in self._prop_dict:
            return self._prop_dict["editPostParameters"]
        else:
            return None

    @edit_post_parameters.setter
    def edit_post_parameters(self, val):
        self._prop_dict["editPostParameters"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..collection_base import CollectionPageBase
from ..model.share import Share


class SharesCollectionPage(CollectionPageBase):

    def __getitem__(self, index):
        """Get the Share at the index specified
        
        Args:
            index (int): The index of the item to get from the SharesCollectionPage

        Returns:
            :class:`Share<onedrivesdk.model.share.Share>`:
                The Share at the index
        """
        return Share(self._prop_list[index])

    def shares(self):
        """Get a generator of Share within the SharesCollectionPage
        
        Yields:
            :class:`Share<onedrivesdk.model.share.Share>`:
                The next Share in the collection
        """
        for item in self._prop_list:
            yield Share(item)



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..collection_base import CollectionPageBase
from ..model.subscription import Subscription


class SubscriptionsCollectionPage(CollectionPageBase):

    def __getitem__(self, index):
        """Get the Subscription at the index specified
        
        Args:
            index (int): The index of the item to get from the SubscriptionsCollectionPage

        Returns:
            :class:`Subscription<onedrivesdk.model.subscription.Subscription>`:
                The Subscription at the index
        """
        return Subscription(self._prop_list[index])

    def subscriptions(self):
        """Get a generator of Subscription within the SubscriptionsCollectionPage
        
        Yields:
            :class:`Subscription<onedrivesdk.model.subscription.Subscription>`:
                The next Subscription in the collection
        """
        for item in self._prop_list:
            yield Subscription(item)



'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''
from __future__ import with_statement
from .model.async_operation_status import AsyncOperationStatus
from .model.item import Item
from .request_base import RequestBase
import threading
import json


class AsyncOperationMonitor(RequestBase):

    def __init__(self, request_url, client, options):
        """Initializes the AsyncOperationMonitor

        Args:
            request_url (str): The request URL to ping for the
                :class:`AsyncOperationStatus<onedrivesdk.models.async_operation_status.AsyncOperationStatus>`
            client (:class:`OneDriveClient<onedrivesdk.requests.one_drive_client.OneDriveClient>`): 
                The client to use for the requests
            options (list): A list of 
                :class:`Option<onedrivesdk.options.Option>`
                to send with the request
        """
        super(AsyncOperationMonitor, self).__init__(request_url,
                                                    client,
                                                    options)
        self._lock = threading.Lock()
        self.method = "GET"
        self._completed = False
        self._has_errors = False
        self._item = None
        self._status = None

        self._start_poll_on_thread()

    def _start_poll_on_thread(self):
        """Begins the polling on a seperate thread"""
        t = threading.Thread(target=self._polling_loop)
        t.start()

    def _stop_poll_on_thread(self):
        """Stops polling on a seperate thread"""
        self._completed = True

    def _polling_loop(self):
        """This method to run within a new thread (as to not block).
        This method will send requests to the status URL until
        the operation has completed (or failed). The AsyncOperationStatus
        will update on each poll. Upon completion, the item (or errors)
        will be populated.
        """
        while not self.completed:
            response = self.send()
            if response.content:
                response_dict = json.loads(response.content)

                with self._lock:
                    self._status_code = response.status

                    if "status" not in response_dict:
                        self._completed = True
                    else:
                        self._status = AsyncOperationStatus(response_dict)
                        if self._status.status == AsyncOperationStatusType.Failed or self._status.status == AsyncOperationStatusType.DeleteFailed:
                            self._completed = True
                            self._has_errors = True

                    if self._completed:
                        self._item = None if self._has_errors else Item(response_dict)

    def poll_until_done(self):
        """Method blocks until done polling for the result 
        of the async operation is returned

        Returns:
            :class:`Item<onedrivesdk.models.item.Item>`: 
                The item from the async operation
        """
        while not self.completed:
            pass
        return self.item

    @property
    def status(self):
        """The status of the async operation
        
        Returns:
            :class:`AsyncOperationStatus<onedrivesdk.models.async_operation_status.AsyncOperationStatus>`:
                The async operation status
        """
        with self._lock:
            return self._status

    @property
    def item(self):
        """Result of the async operation will be placed here when
        polling is completed

        Returns:
            :class:`Item<onedrivesdk.models.item.Item>`:
                The item from async operation
        """
        with self._lock:
            return self._item

    @property
    def completed(self):
        """Whether or not the async operation has completed

        Returns:
            bool: Whether or not the async operation has completed
        """
        with self._lock:
            return self._completed

    @property
    def has_errors(self):
        """Whether or not the async operation has errors
        
        Returns:
            bool: Whether or not the async operation has errors
        """
        with self._lock:
            return self._has_errors


class AsyncOperationStatusType(object):
    """Various status types that can be found in
    the :class:`onedrivesdk.models.async_operation_status.AsyncOperationStatus` status
    """

    #: The operation has not started yet
    NotStarted = "notStarted"
    #: The operation is in progress
    InProgress = "inProgress"
    #: The operation has completed
    Completed = "completed"
    #: The operation is updating
    Updating = "updating"
    #: The operation has failed
    Failed = "failed"
    #: The delete is pending
    DeletePending = "deletePending"
    #: The delete has failed
    DeleteFailed = "deleteFailed"
    #: The operation is waiting to start
    Waiting = "waiting"



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.identity_set import IdentitySet
from ..one_drive_object_base import OneDriveObjectBase


class Shared(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def effective_roles(self):
        """Gets and sets the effectiveRoles
        
        Returns: 
            str:
                The effectiveRoles
        """
        if "effectiveRoles" in self._prop_dict:
            return self._prop_dict["effectiveRoles"]
        else:
            return None

    @effective_roles.setter
    def effective_roles(self, val):
        self._prop_dict["effectiveRoles"] = val

    @property
    def owner(self):
        """
        Gets and sets the owner
        
        Returns: 
            :class:`IdentitySet<onedrivesdk.model.identity_set.IdentitySet>`:
                The owner
        """
        if "owner" in self._prop_dict:
            if isinstance(self._prop_dict["owner"], OneDriveObjectBase):
                return self._prop_dict["owner"]
            else :
                self._prop_dict["owner"] = IdentitySet(self._prop_dict["owner"])
                return self._prop_dict["owner"]

        return None

    @owner.setter
    def owner(self, val):
        self._prop_dict["owner"] = val
    @property
    def scope(self):
        """Gets and sets the scope
        
        Returns: 
            str:
                The scope
        """
        if "scope" in self._prop_dict:
            return self._prop_dict["scope"]
        else:
            return None

    @scope.setter
    def scope(self, val):
        self._prop_dict["scope"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) 2015 Microsoft Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.item_delta_collection_page import ItemDeltaCollectionPage
from ..request.items_collection import ItemsCollectionResponse


class ItemDeltaCollectionResponse(ItemsCollectionResponse):

    @property
    def collection_page(self):
        """The collection page stored in the response JSON
        
        Returns:
            :class:`ItemDeltaCollectionPage<onedrivesdk.request.item_delta_collection.ItemDeltaCollectionPage>`:
                The collection page
        """
        token = self._prop_dict["@delta.token"] if "@delta.token" in self._prop_dict else None
        delta_link = self._prop_dict["@odata.deltaLink"] if "@odata.deltaLink" in self._prop_dict else None
        next_page_link = self._prop_dict["@odata.nextLink"] if "@odata.nextLink" in self._prop_dict else None

        if self._collection_page:
            self._collection_page._prop_list = self._prop_dict["value"]
            self._collection_page._token = token
            self._collection_page._delta_link = delta_link
            self._collection_page._next_page_link = next_page_link
        else:
            self._collection_page = ItemDeltaCollectionPage(self._prop_dict["value"],
                                                            token,
                                                            delta_link,
                                                            next_page_link)

        return self._collection_page

from ..request.item_delta import ItemDeltaRequest


# -*- coding: utf-8 -*- 
'''
# Copyright (c) 2015 Microsoft Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..request.drives_collection import DrivesCollectionRequestBuilder
from ..request.shares_collection import SharesCollectionRequestBuilder
import sys


class OneDriveClient(object):

    def __init__(self, base_url, auth_provider, http_provider, loop=None):
        """Initialize the :class:`OneDriveClient` to be
            used for all OneDrive API interactions
        
        Args:
            base_url (str): The OneDrive base url to use for API interactions
            auth_provider(:class:`AuthProviderBase<onedrivesdk.auth_provider_base.AuthProviderBase>`):
                The authentication provider used by the client to auth
                with OneDrive services
            http_provider(:class:`HttpProviderBase<onedrivesdk.http_provider_base.HttpProviderBase>`):
                The HTTP provider used by the client to send all 
                requests to OneDrive
            loop (BaseEventLoop): Default to None, the AsyncIO loop 
                to use for all async requests
        """
        self.base_url = base_url
        self.auth_provider = auth_provider
        self.http_provider = http_provider

        if sys.version_info >= (3, 4, 0):
            import asyncio
            self._loop = loop if loop else asyncio.get_event_loop()

    @property
    def auth_provider(self):
        """Gets and sets the client auth provider
        
        Returns:
            :class:`AuthProviderBase<onedrivesdk.auth_provider_base.AuthProviderBase>`: 
            The authentication provider
        """
        return self._auth_provider

    @auth_provider.setter
    def auth_provider(self, value):
        self._auth_provider = value

    @property
    def http_provider(self):
        """Gets and sets the client HTTP provider

        Returns: 
            :class:`HttpProviderBase<onedrivesdk.http_provider_base.HttpProviderBase>`: 
                The HTTP provider
        """
        return self._http_provider

    @http_provider.setter
    def http_provider(self, value):
        self._http_provider = value

    @property
    def base_url(self):
        """Gets and sets the base URL used by the client to make requests

        Returns:
            str: The base URL
        """
        return self._base_url

    @base_url.setter
    def base_url(self, value):
        self._base_url = value

    @property
    def drives(self):
        """Get the DrivesCollectionRequestBuilder for constructing requests
        
        Returns:
            :class:`DrivesCollectionRequestBuilder<onedrivesdk.request.drives_collection.DrivesCollectionRequestBuilder>`:
                The DrivesCollectionRequestBuilder to return
        """
        return DrivesCollectionRequestBuilder(self.base_url + "drives", self)

    @property
    def shares(self):
        """Get the SharesCollectionRequestBuilder for constructing requests
        
        Returns:
            :class:`SharesCollectionRequestBuilder<onedrivesdk.request.shares_collection.SharesCollectionRequestBuilder>`:
                The SharesCollectionRequestBuilder to return
        """
        return SharesCollectionRequestBuilder(self.base_url + "shares", self)



'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''
from __future__ import generators, unicode_literals
from .version import __version__
from .options import *
from .one_drive_object_base import OneDriveObjectBase

try:
    from urllib.parse import urlparse, parse_qsl, urlencode, urlunparse
except ImportError:
    from urlparse import urlparse, parse_qsl, urlunparse
    from urllib import urlencode


class RequestBase(object):

    def __init__(self, request_url, client, options):
        """Initialize a request to be sent

        Args:
            request_url (str): The URL to send the request to
            client (:class:`OneDriveClient<onedrivesdk.requests.one_drive_client.OneDriveClient>`):
                The client used to make the requests
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to add to the request
        """
        self._client = client
        self._request_url = request_url
        self._headers = {}
        self._query_options = {}
        self.content_type = None

        if (options):
            header_list = [
                pair for pair in options if isinstance(pair, HeaderOption)]
            self._headers = {pair.key: pair.value for pair in header_list}

            query_list = [
                pair for pair in options if isinstance(pair, QueryOption)]
            self._query_options = {pair.key: pair.value for pair in query_list}

    @property
    def request_url(self):
        """Gets the request URL with query string appended

        Returns:
            str: The request URL
        """
        url_parts = list(urlparse(self._request_url))
        query_dict = dict(parse_qsl(url_parts[4]))
        self._query_options.update(query_dict)
        url_parts[4] = urlencode(self._query_options)
        return urlunparse(url_parts)

    @property
    def content_type(self):
        """Gets and sets the content-type header of the request
        (ex. application/x-www-form-urlencoded)

        Returns:
            str: The request content-type
        """
        return self._content_type

    @content_type.setter
    def content_type(self, value):
        self._content_type = value

    @property
    def method(self):
        """Gets and sets the HTTP method by which to send the
        request (ex. PUT, GET)

        Returns:
            str: The HTTP method
        """
        return self._method

    @method.setter
    def method(self, value):
        self._method = value

    def append_option(self, option):
        """Appends an option to the request

        Args:
            option (:class:`Option<onedrivesdk.options.Option>`):
                The option to append to the request
        """
        if isinstance(option, HeaderOption):
            self._headers[option.key] = option.value
        elif isinstance(option, QueryOption):
            self._query_options[option.key] = option.value

    def send(self, content=None, path=None, data=None):
        """Send the request using the client specified
        at request initialization

        Args:
            content (str): Defaults to None, the body of the request
                that will be sent
            path (str): Defaults to None, the local path of the file which
                will be sent
            data (file object): Defaults to none, the file object of the
                file which will be sent

        Returns:
            :class:`HttpResponse<onedrivesdk.http_response.HttpResponse>`:
                The response to the request
        """
        self._client.auth_provider.authenticate_request(self)

        self.append_option(HeaderOption("X-RequestStats",
                                        "SDK-Version=python-v"+__version__))

        if self.content_type:
            self.append_option(HeaderOption("Content-Type", self.content_type))

        if path:
            response = self._client.http_provider.send(
                self.method,
                self._headers,
                self.request_url,
                path=path)
        elif data:
            response = self._client.http_provider.send(
                self.method,
                self._headers,
                self.request_url,
                data=data)
        else:
            content_dict = None

            if content:
                content_dict = content.to_dict() if isinstance(
                    content, OneDriveObjectBase) else content

            response = self._client.http_provider.send(
                self.method,
                self._headers,
                self.request_url,
                content=content_dict)

        return response

    def download_item(self, path):
        """Download a file to a local path

        Args:
            path (str): The local path to download the file

        Returns:
            :class:`HttpResponse<onedrivesdk.http_response.HttpResponse>`:
                The response to the request 
        """
        self._client.auth_provider.authenticate_request(self)

        self.append_option(HeaderOption("X-RequestStats",
                                        "SDK-Version=python-v"+__version__))

        if self.content_type:
            self.append_option(HeaderOption("Content-Type", self.content_type))

        response = self._client.http_provider.download(
            self._headers,
            self.request_url,
            path)

        return response

    def _set_query_options(self, expand=None, select=None, top=None, order_by=None):
        """Adds query options from a set of known parameters

        Args:
            expand (str): Default None, comma-seperated list of relationships
                to expand in the response.
            select (str): Default None, comma-seperated list of properties to
                include in the response.
            top (int): Default None, the number of items to return in a result.
            order_by (str): Default None, comma-seperated list of properties
                that are used to sort the order of items in the response.
        """
        if expand:
            self.append_option(QueryOption("expand", expand))

        if select:
            self.append_option(QueryOption("select", select))

        if top:
            self.append_option(QueryOption("top", top))

        if order_by:
            self.append_option(QueryOption("orderby", order_by))



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from .tag_request import TagRequest
from ..request_builder_base import RequestBuilderBase
import asyncio


class TagRequestBuilder(RequestBuilderBase):

    def __init__(self, request_url, client):
        """Initialize the TagRequestBuilder

        Args:
            request_url (str): The url to perform the TagRequest
                on
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
        """
        super(TagRequestBuilder, self).__init__(request_url, client)

    def request(self, expand=None, select=None, options=None):
        """Builds the TagRequest

        Args:
            expand (str): Default None, comma-seperated list of relationships
                to expand in the response.
            select (str): Default None, comma-seperated list of properties to
                include in the response.
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request. Defaults to None.

        Returns:
            :class:`TagRequest<onedrivesdk.request.tag_request.TagRequest>`:
                The TagRequest
        """
        req = TagRequest(self._request_url, self._client, options)
        req._set_query_options(expand=expand, select=select)
        return req

    def delete(self):
        """Deletes the specified Tag."""
        self.request().delete()

    @asyncio.coroutine
    def delete_async(self):
        """Deletes the specified Tag."""
        yield from self.request().delete_async()
    def get(self):
        """Gets the specified Tag.
        
        Returns:
            :class:`Tag<onedrivesdk.model.tag.Tag>`:
                The Tag.
        """
        return self.request().get()

    @asyncio.coroutine
    def get_async(self):
        """Gets the specified Tag in async.

        Returns:
            :class:`Tag<onedrivesdk.model.tag.Tag>`:
                The Tag.
        """
        entity = yield from self.request().get_async()
        return entity
    def update(self, tag):
        """Updates the specified Tag.
        
        Args:
            tag (:class:`Tag<onedrivesdk.model.tag.Tag>`):
                The Tag to update.

        Returns:
            :class:`Tag<onedrivesdk.model.tag.Tag>`:
                The updated Tag
        """
        return self.request().update(tag)

    @asyncio.coroutine
    def update_async(self, tag):
        """Updates the specified Tag in async
        
        Args:
            tag (:class:`Tag<onedrivesdk.model.tag.Tag>`):
                The Tag to update.

        Returns:
            :class:`Tag<onedrivesdk.model.tag.Tag>`:
                The updated Tag.
        """
        entity = yield from self.request().update_async(tag)
        return entity



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..one_drive_object_base import OneDriveObjectBase


class Deleted(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def state(self):
        """Gets and sets the state
        
        Returns: 
            str:
                The state
        """
        if "state" in self._prop_dict:
            return self._prop_dict["state"]
        else:
            return None

    @state.setter
    def state(self, val):
        self._prop_dict["state"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from ..request_base import RequestBase
from ..model.item import Item
import json

class ItemRequest(RequestBase):
    """The type ItemRequest."""
    
    def __init__(self, request_url, client, options):
        """Constructs a new ItemRequest.

        Args:
            request_url (str): The url to perform the ItemRequest
                on
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request
        """
        super(ItemRequest, self).__init__(request_url, client, options)

    def delete(self):
        """Deletes the specified Item."""
        self.method = "DELETE"
        self.send()

    def get(self):
        """Gets the specified Item.
        
        Returns:
            :class:`Item<onedrivesdk.model.item.Item>`:
                The Item.
        """
        self.method = "GET"
        entity = Item(json.loads(self.send().content))
        self._initialize_collection_properties(entity)
        return entity

    def update(self, item):
        """Updates the specified Item.
        
        Args:
            item (:class:`Item<onedrivesdk.model.item.Item>`):
                The Item to update.

        Returns:
            :class:`Item<onedrivesdk.model.item.Item>`:
                The updated Item.
        """
        self.content_type = "application/json"
        self.method = "PATCH"
        entity = Item(json.loads(self.send(item).content))
        self._initialize_collection_properties(entity)
        return entity

    def _initialize_collection_properties(self, value):
        if value and value._prop_dict:
            if value.permissions:
                if "permissions@odata.nextLink" in value._prop_dict:
                    next_page_link = value._prop_dict["permissions@odata.nextLink"]
                    value.permissions._next_page_link = next_page_link
            if value.subscriptions:
                if "subscriptions@odata.nextLink" in value._prop_dict:
                    next_page_link = value._prop_dict["subscriptions@odata.nextLink"]
                    value.subscriptions._next_page_link = next_page_link
            if value.versions:
                if "versions@odata.nextLink" in value._prop_dict:
                    next_page_link = value._prop_dict["versions@odata.nextLink"]
                    value.versions._next_page_link = next_page_link
            if value.children:
                if "children@odata.nextLink" in value._prop_dict:
                    next_page_link = value._prop_dict["children@odata.nextLink"]
                    value.children._next_page_link = next_page_link
            if value.tags:
                if "tags@odata.nextLink" in value._prop_dict:
                    next_page_link = value._prop_dict["tags@odata.nextLink"]
                    value.tags._next_page_link = next_page_link
            if value.thumbnails:
                if "thumbnails@odata.nextLink" in value._prop_dict:
                    next_page_link = value._prop_dict["thumbnails@odata.nextLink"]
                    value.thumbnails._next_page_link = next_page_link



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..collection_base import CollectionRequestBase, CollectionResponseBase
from ..request_builder_base import RequestBuilderBase
from ..model.thumbnails_collection_page import ThumbnailsCollectionPage
import json

class ThumbnailsCollectionRequest(CollectionRequestBase):

    def __init__(self, request_url, client, options):
        """Initialize the ThumbnailsCollectionRequest
        
        Args:
            request_url (str): The url to perform the ThumbnailsCollectionRequest
                on
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request
        """
        super(ThumbnailsCollectionRequest, self).__init__(request_url, client, options)

    def get(self):
        """Gets the ThumbnailsCollectionPage

        Returns: 
            :class:`ThumbnailsCollectionPage<onedrivesdk.model.thumbnails_collection_page.ThumbnailsCollectionPage>`:
                The ThumbnailsCollectionPage
        """
        self.method = "GET"
        collection_response = ThumbnailsCollectionResponse(json.loads(self.send().content))
        return self._page_from_response(collection_response)


    @staticmethod
    def get_next_page_request(collection_page, client, options=None):
        """Gets the ThumbnailsCollectionRequest for the next page. Returns None if there is no next page

        Args:
            collection_page (:class:`ThumbnailsCollectionPage<onedrivesdk.model.thumbnails_collection_page.ThumbnailsCollectionPage>`):
                The collection to get the next page for
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request. Defaults to None.

        Yields: 
            :class:`ThumbnailsCollectionRequest<onedrivesdk.request.thumbnails_collection.ThumbnailsCollectionRequest>`:
                The ThumbnailsCollectionRequest
        """
        if collection_page._next_page_link:
            return ThumbnailsCollectionRequest(collection_page._next_page_link, client, options)
        else:
            return None

class ThumbnailsCollectionRequestBuilder(RequestBuilderBase):

    def __getitem__(self, key):
        """Get the ThumbnailSetRequestBuilder with the specified key
        
        Args:
            key (str): The key to get a ThumbnailSetRequestBuilder for
        
        Returns: 
            :class:`ThumbnailSetRequestBuilder<onedrivesdk.request.thumbnail_set_request_builder.ThumbnailSetRequestBuilder>`:
                A ThumbnailSetRequestBuilder for that key
        """
        return ThumbnailSetRequestBuilder(self.append_to_request_url(str(key)), self._client)

    def request(self, expand=None, select=None, top=None, order_by=None, options=None):
        """Builds the ThumbnailsCollectionRequest
        
        Args:
            expand (str): Default None, comma-seperated list of relationships
                to expand in the response.
            select (str): Default None, comma-seperated list of properties to
                include in the response.
            top (int): Default None, the number of items to return in a result.
            order_by (str): Default None, comma-seperated list of properties
                that are used to sort the order of items in the response.
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request. Defaults to None.

        Returns:
            :class:`ThumbnailsCollectionRequest<onedrivesdk.request.thumbnails_collection.ThumbnailsCollectionRequest>`:
                The ThumbnailsCollectionRequest
        """
        req = ThumbnailsCollectionRequest(self._request_url, self._client, options)
        req._set_query_options(expand=expand, select=select, top=top, order_by=order_by)
        return req

    def get(self):
        """Gets the ThumbnailsCollectionPage

        Returns: 
            :class:`ThumbnailsCollectionPage<onedrivesdk.model.thumbnails_collection_page.ThumbnailsCollectionPage>`:
                The ThumbnailsCollectionPage
        """
        return self.request().get()



class ThumbnailsCollectionResponse(CollectionResponseBase):

    @property
    def collection_page(self):
        """The collection page stored in the response JSON
        
        Returns:
            :class:`ThumbnailsCollectionPage<onedrivesdk.model.thumbnails_collection_page.ThumbnailsCollectionPage>`:
                The collection page
        """
        if self._collection_page:
            self._collection_page._prop_list = self._prop_dict["value"]
        else:
            self._collection_page = ThumbnailsCollectionPage(self._prop_dict["value"])

        return self._collection_page


from ..request.thumbnail_set_request_builder import ThumbnailSetRequestBuilder



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from .drive_request import DriveRequest
from ..request_builder_base import RequestBuilderBase
from ..request.drive_recent import DriveRecentRequestBuilder
import asyncio


class DriveRequestBuilder(RequestBuilderBase):

    def __init__(self, request_url, client):
        """Initialize the DriveRequestBuilder

        Args:
            request_url (str): The url to perform the DriveRequest
                on
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
        """
        super(DriveRequestBuilder, self).__init__(request_url, client)

    def request(self, expand=None, select=None, options=None):
        """Builds the DriveRequest

        Args:
            expand (str): Default None, comma-seperated list of relationships
                to expand in the response.
            select (str): Default None, comma-seperated list of properties to
                include in the response.
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request. Defaults to None.

        Returns:
            :class:`DriveRequest<onedrivesdk.request.drive_request.DriveRequest>`:
                The DriveRequest
        """
        req = DriveRequest(self._request_url, self._client, options)
        req._set_query_options(expand=expand, select=select)
        return req

    def delete(self):
        """Deletes the specified Drive."""
        self.request().delete()

    @asyncio.coroutine
    def delete_async(self):
        """Deletes the specified Drive."""
        yield from self.request().delete_async()
    def get(self):
        """Gets the specified Drive.
        
        Returns:
            :class:`Drive<onedrivesdk.model.drive.Drive>`:
                The Drive.
        """
        return self.request().get()

    @asyncio.coroutine
    def get_async(self):
        """Gets the specified Drive in async.

        Returns:
            :class:`Drive<onedrivesdk.model.drive.Drive>`:
                The Drive.
        """
        entity = yield from self.request().get_async()
        return entity
    def update(self, drive):
        """Updates the specified Drive.
        
        Args:
            drive (:class:`Drive<onedrivesdk.model.drive.Drive>`):
                The Drive to update.

        Returns:
            :class:`Drive<onedrivesdk.model.drive.Drive>`:
                The updated Drive
        """
        return self.request().update(drive)

    @asyncio.coroutine
    def update_async(self, drive):
        """Updates the specified Drive in async
        
        Args:
            drive (:class:`Drive<onedrivesdk.model.drive.Drive>`):
                The Drive to update.

        Returns:
            :class:`Drive<onedrivesdk.model.drive.Drive>`:
                The updated Drive.
        """
        entity = yield from self.request().update_async(drive)
        return entity

    @property
    def items(self):
        """The items for the DriveRequestBuilder

        Returns: 
            :class:`ItemsCollectionRequestBuilder<onedrivesdk.request.items_collection.ItemsCollectionRequestBuilder>`:
                A request builder created from the DriveRequestBuilder
        """
        return ItemsCollectionRequestBuilder(self.append_to_request_url("items"), self._client)

    @property
    def shared(self):
        """The shared for the DriveRequestBuilder

        Returns: 
            :class:`SharedCollectionRequestBuilder<onedrivesdk.request.shared_collection.SharedCollectionRequestBuilder>`:
                A request builder created from the DriveRequestBuilder
        """
        return SharedCollectionRequestBuilder(self.append_to_request_url("shared"), self._client)

    @property
    def special(self):
        """The special for the DriveRequestBuilder

        Returns: 
            :class:`SpecialCollectionRequestBuilder<onedrivesdk.request.special_collection.SpecialCollectionRequestBuilder>`:
                A request builder created from the DriveRequestBuilder
        """
        return SpecialCollectionRequestBuilder(self.append_to_request_url("special"), self._client)
    def recent(self):
        """Executes the recent method


        Returns:
            :class:`DriveRecentRequestBuilder<onedrivesdk.request.drive_recent.DriveRecentRequestBuilder>`:
                A DriveRecentRequestBuilder for the method
        """
        return DriveRecentRequestBuilder(self.append_to_request_url("recent"), self._client)

from ..request.items_collection import ItemsCollectionRequestBuilder
from ..request.shared_collection import SharedCollectionRequestBuilder
from ..request.special_collection import SpecialCollectionRequestBuilder



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
'''

from .model.item_reference import ItemReference
from .model.recipients import Recipients
from .model.upload_session import UploadSession
from .model.chunked_upload_session_descriptor import ChunkedUploadSessionDescriptor
from .model.audio import Audio
from .model.async_operation_status import AsyncOperationStatus
from .model.auto_tagged import AutoTagged
from .model.status import Status
from .model.deleted import Deleted
from .model.file import File
from .model.file_system_info import FileSystemInfo
from .model.folder import Folder
from .model.hashes import Hashes
from .model.image import Image
from .model.location import Location
from .model.open_with_set import OpenWithSet
from .model.photo import Photo
from .model.quota import Quota
from .model.search_result import SearchResult
from .model.sharing_invitation import SharingInvitation
from .model.shared import Shared
from .model.sharing_link import SharingLink
from .model.special_folder import SpecialFolder
from .model.video import Video
from .model.identity_set import IdentitySet
from .model.identity import Identity
from .model.open_with_app import OpenWithApp
from .model.thumbnail import Thumbnail
from .model.storage_plans import StoragePlans
from .model.drive import Drive
from .model.share import Share
from .model.item import Item
from .model.permission import Permission
from .model.subscription import Subscription
from .model.tag import Tag
from .model.thumbnail_set import ThumbnailSet
from .request.drive_request import DriveRequest
from .request.drive_request_builder import DriveRequestBuilder
from .request.share_request import ShareRequest
from .request.share_request_builder import ShareRequestBuilder
from .request.item_request import ItemRequest
from .request.item_request_builder import ItemRequestBuilder
from .request.permission_request import PermissionRequest
from .request.permission_request_builder import PermissionRequestBuilder
from .request.subscription_request import SubscriptionRequest
from .request.subscription_request_builder import SubscriptionRequestBuilder
from .request.tag_request import TagRequest
from .request.tag_request_builder import TagRequestBuilder
from .request.thumbnail_set_request import ThumbnailSetRequest
from .request.thumbnail_set_request_builder import ThumbnailSetRequestBuilder
from .request.thumbnail_request import ThumbnailRequest
from .request.thumbnail_request_builder import ThumbnailRequestBuilder
from .request.items_collection import ItemsCollectionRequest, ItemsCollectionRequestBuilder, ItemsCollectionResponse
from .model.items_collection_page import ItemsCollectionPage
from .request.shared_collection import SharedCollectionRequest, SharedCollectionRequestBuilder, SharedCollectionResponse
from .model.shared_collection_page import SharedCollectionPage
from .request.special_collection import SpecialCollectionRequest, SpecialCollectionRequestBuilder, SpecialCollectionResponse
from .model.special_collection_page import SpecialCollectionPage
from .request.items_collection import ItemsCollectionRequest, ItemsCollectionRequestBuilder, ItemsCollectionResponse
from .model.items_collection_page import ItemsCollectionPage
from .request.permissions_collection import PermissionsCollectionRequest, PermissionsCollectionRequestBuilder, PermissionsCollectionResponse
from .model.permissions_collection_page import PermissionsCollectionPage
from .request.subscriptions_collection import SubscriptionsCollectionRequest, SubscriptionsCollectionRequestBuilder, SubscriptionsCollectionResponse
from .model.subscriptions_collection_page import SubscriptionsCollectionPage
from .request.versions_collection import VersionsCollectionRequest, VersionsCollectionRequestBuilder, VersionsCollectionResponse
from .model.versions_collection_page import VersionsCollectionPage
from .request.children_collection import ChildrenCollectionRequest, ChildrenCollectionRequestBuilder, ChildrenCollectionResponse
from .model.children_collection_page import ChildrenCollectionPage
from .request.tags_collection import TagsCollectionRequest, TagsCollectionRequestBuilder, TagsCollectionResponse
from .model.tags_collection_page import TagsCollectionPage
from .request.thumbnails_collection import ThumbnailsCollectionRequest, ThumbnailsCollectionRequestBuilder, ThumbnailsCollectionResponse
from .model.thumbnails_collection_page import ThumbnailsCollectionPage
from .request.drives_collection import DrivesCollectionRequest, DrivesCollectionRequestBuilder, DrivesCollectionResponse
from .model.drives_collection_page import DrivesCollectionPage
from .request.shares_collection import SharesCollectionRequest, SharesCollectionRequestBuilder, SharesCollectionResponse
from .model.shares_collection_page import SharesCollectionPage
from .request.drive_recent import DriveRecentRequest
from .request.item_create_session import ItemCreateSessionRequest
from .request.item_copy import ItemCopyRequest
from .request.item_create_link import ItemCreateLinkRequest
from .request.item_invite import ItemInviteRequest
from .request.item_delta import ItemDeltaRequest
from .request.item_search import ItemSearchRequest
from .request.item_delta_collection import ItemDeltaCollectionResponse
from .model.item_delta_collection_page import ItemDeltaCollectionPage
from .request.item_content_request import ItemContentRequest, ItemContentRequestBuilder
from .request.thumbnail_content_request import ThumbnailContentRequest, ThumbnailContentRequestBuilder
from .request.one_drive_client import OneDriveClient
from .auth_provider import AuthProvider
from .http_provider import HttpProvider
from .extensions.onedrivesdk_helper import *
from .extensions import *
import sys
if sys.version_info >= (3, 4, 0):
    from .version_bridge import *



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..one_drive_object_base import OneDriveObjectBase


class Recipients(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def email(self):
        """Gets and sets the email
        
        Returns: 
            str:
                The email
        """
        if "email" in self._prop_dict:
            return self._prop_dict["email"]
        else:
            return None

    @email.setter
    def email(self, val):
        self._prop_dict["email"] = val

    @property
    def alias(self):
        """Gets and sets the alias
        
        Returns: 
            str:
                The alias
        """
        if "alias" in self._prop_dict:
            return self._prop_dict["alias"]
        else:
            return None

    @alias.setter
    def alias(self, val):
        self._prop_dict["alias"] = val

    @property
    def object_id(self):
        """Gets and sets the objectId
        
        Returns: 
            str:
                The objectId
        """
        if "objectId" in self._prop_dict:
            return self._prop_dict["objectId"]
        else:
            return None

    @object_id.setter
    def object_id(self, val):
        self._prop_dict["objectId"] = val

    @property
    def permission_identity_type_input(self):
        """Gets and sets the permissionIdentityTypeInput
        
        Returns: 
            str:
                The permissionIdentityTypeInput
        """
        if "permissionIdentityTypeInput" in self._prop_dict:
            return self._prop_dict["permissionIdentityTypeInput"]
        else:
            return None

    @permission_identity_type_input.setter
    def permission_identity_type_input(self, val):
        self._prop_dict["permissionIdentityTypeInput"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..one_drive_object_base import OneDriveObjectBase


class Audio(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def album(self):
        """Gets and sets the album
        
        Returns: 
            str:
                The album
        """
        if "album" in self._prop_dict:
            return self._prop_dict["album"]
        else:
            return None

    @album.setter
    def album(self, val):
        self._prop_dict["album"] = val

    @property
    def album_artist(self):
        """Gets and sets the albumArtist
        
        Returns: 
            str:
                The albumArtist
        """
        if "albumArtist" in self._prop_dict:
            return self._prop_dict["albumArtist"]
        else:
            return None

    @album_artist.setter
    def album_artist(self, val):
        self._prop_dict["albumArtist"] = val

    @property
    def artist(self):
        """Gets and sets the artist
        
        Returns: 
            str:
                The artist
        """
        if "artist" in self._prop_dict:
            return self._prop_dict["artist"]
        else:
            return None

    @artist.setter
    def artist(self, val):
        self._prop_dict["artist"] = val

    @property
    def bitrate(self):
        """Gets and sets the bitrate
        
        Returns: 
            int:
                The bitrate
        """
        if "bitrate" in self._prop_dict:
            return self._prop_dict["bitrate"]
        else:
            return None

    @bitrate.setter
    def bitrate(self, val):
        self._prop_dict["bitrate"] = val

    @property
    def composers(self):
        """Gets and sets the composers
        
        Returns: 
            str:
                The composers
        """
        if "composers" in self._prop_dict:
            return self._prop_dict["composers"]
        else:
            return None

    @composers.setter
    def composers(self, val):
        self._prop_dict["composers"] = val

    @property
    def copyright(self):
        """Gets and sets the copyright
        
        Returns: 
            str:
                The copyright
        """
        if "copyright" in self._prop_dict:
            return self._prop_dict["copyright"]
        else:
            return None

    @copyright.setter
    def copyright(self, val):
        self._prop_dict["copyright"] = val

    @property
    def disc(self):
        """Gets and sets the disc
        
        Returns: 
            int:
                The disc
        """
        if "disc" in self._prop_dict:
            return self._prop_dict["disc"]
        else:
            return None

    @disc.setter
    def disc(self, val):
        self._prop_dict["disc"] = val

    @property
    def disc_count(self):
        """Gets and sets the discCount
        
        Returns: 
            int:
                The discCount
        """
        if "discCount" in self._prop_dict:
            return self._prop_dict["discCount"]
        else:
            return None

    @disc_count.setter
    def disc_count(self, val):
        self._prop_dict["discCount"] = val

    @property
    def duration(self):
        """Gets and sets the duration
        
        Returns: 
            int:
                The duration
        """
        if "duration" in self._prop_dict:
            return self._prop_dict["duration"]
        else:
            return None

    @duration.setter
    def duration(self, val):
        self._prop_dict["duration"] = val

    @property
    def genre(self):
        """Gets and sets the genre
        
        Returns: 
            str:
                The genre
        """
        if "genre" in self._prop_dict:
            return self._prop_dict["genre"]
        else:
            return None

    @genre.setter
    def genre(self, val):
        self._prop_dict["genre"] = val

    @property
    def has_drm(self):
        """Gets and sets the hasDrm
        
        Returns: 
            bool:
                The hasDrm
        """
        if "hasDrm" in self._prop_dict:
            return self._prop_dict["hasDrm"]
        else:
            return None

    @has_drm.setter
    def has_drm(self, val):
        self._prop_dict["hasDrm"] = val

    @property
    def is_variable_bitrate(self):
        """Gets and sets the isVariableBitrate
        
        Returns: 
            bool:
                The isVariableBitrate
        """
        if "isVariableBitrate" in self._prop_dict:
            return self._prop_dict["isVariableBitrate"]
        else:
            return None

    @is_variable_bitrate.setter
    def is_variable_bitrate(self, val):
        self._prop_dict["isVariableBitrate"] = val

    @property
    def title(self):
        """Gets and sets the title
        
        Returns: 
            str:
                The title
        """
        if "title" in self._prop_dict:
            return self._prop_dict["title"]
        else:
            return None

    @title.setter
    def title(self, val):
        self._prop_dict["title"] = val

    @property
    def track(self):
        """Gets and sets the track
        
        Returns: 
            int:
                The track
        """
        if "track" in self._prop_dict:
            return self._prop_dict["track"]
        else:
            return None

    @track.setter
    def track(self, val):
        self._prop_dict["track"] = val

    @property
    def track_count(self):
        """Gets and sets the trackCount
        
        Returns: 
            int:
                The trackCount
        """
        if "trackCount" in self._prop_dict:
            return self._prop_dict["trackCount"]
        else:
            return None

    @track_count.setter
    def track_count(self, val):
        self._prop_dict["trackCount"] = val

    @property
    def year(self):
        """Gets and sets the year
        
        Returns: 
            int:
                The year
        """
        if "year" in self._prop_dict:
            return self._prop_dict["year"]
        else:
            return None

    @year.setter
    def year(self, val):
        self._prop_dict["year"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.identity_set import IdentitySet
from ..model.item_reference import ItemReference
from ..model.audio import Audio
from ..model.deleted import Deleted
from ..model.file import File
from ..model.file_system_info import FileSystemInfo
from ..model.folder import Folder
from ..model.image import Image
from ..model.location import Location
from ..model.open_with_set import OpenWithSet
from ..model.photo import Photo
from ..model.search_result import SearchResult
from ..model.shared import Shared
from ..model.special_folder import SpecialFolder
from ..model.video import Video
from ..model.permission import Permission
from ..model.subscription import Subscription
from ..model.tag import Tag
from ..model.thumbnail_set import ThumbnailSet
from datetime import datetime
from ..one_drive_object_base import OneDriveObjectBase


class Item(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def created_by(self):
        """
        Gets and sets the createdBy
        
        Returns: 
            :class:`IdentitySet<onedrivesdk.model.identity_set.IdentitySet>`:
                The createdBy
        """
        if "createdBy" in self._prop_dict:
            if isinstance(self._prop_dict["createdBy"], OneDriveObjectBase):
                return self._prop_dict["createdBy"]
            else :
                self._prop_dict["createdBy"] = IdentitySet(self._prop_dict["createdBy"])
                return self._prop_dict["createdBy"]

        return None

    @created_by.setter
    def created_by(self, val):
        self._prop_dict["createdBy"] = val

    @property
    def created_date_time(self):
        """
        Gets and sets the createdDateTime
        
        Returns:
            datetime:
                The createdDateTime
        """
        if "createdDateTime" in self._prop_dict:
            if '.' in self._prop_dict["createdDateTime"]:
                return datetime.strptime(self._prop_dict["createdDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S.%f")
            else:
                return datetime.strptime(self._prop_dict["createdDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S")
        else:
            return None

    @created_date_time.setter
    def created_date_time(self, val):
        self._prop_dict["createdDateTime"] = val.isoformat()+((".0" if val.time().microsecond == 0 else "")+"Z")

    @property
    def c_tag(self):
        """
        Gets and sets the cTag
        
        Returns:
            str:
                The cTag
        """
        if "cTag" in self._prop_dict:
            return self._prop_dict["cTag"]
        else:
            return None

    @c_tag.setter
    def c_tag(self, val):
        self._prop_dict["cTag"] = val

    @property
    def description(self):
        """
        Gets and sets the description
        
        Returns:
            str:
                The description
        """
        if "description" in self._prop_dict:
            return self._prop_dict["description"]
        else:
            return None

    @description.setter
    def description(self, val):
        self._prop_dict["description"] = val

    @property
    def e_tag(self):
        """
        Gets and sets the eTag
        
        Returns:
            str:
                The eTag
        """
        if "eTag" in self._prop_dict:
            return self._prop_dict["eTag"]
        else:
            return None

    @e_tag.setter
    def e_tag(self, val):
        self._prop_dict["eTag"] = val

    @property
    def id(self):
        """
        Gets and sets the id
        
        Returns:
            str:
                The id
        """
        if "id" in self._prop_dict:
            return self._prop_dict["id"]
        else:
            return None

    @id.setter
    def id(self, val):
        self._prop_dict["id"] = val

    @property
    def last_modified_by(self):
        """
        Gets and sets the lastModifiedBy
        
        Returns: 
            :class:`IdentitySet<onedrivesdk.model.identity_set.IdentitySet>`:
                The lastModifiedBy
        """
        if "lastModifiedBy" in self._prop_dict:
            if isinstance(self._prop_dict["lastModifiedBy"], OneDriveObjectBase):
                return self._prop_dict["lastModifiedBy"]
            else :
                self._prop_dict["lastModifiedBy"] = IdentitySet(self._prop_dict["lastModifiedBy"])
                return self._prop_dict["lastModifiedBy"]

        return None

    @last_modified_by.setter
    def last_modified_by(self, val):
        self._prop_dict["lastModifiedBy"] = val

    @property
    def last_modified_date_time(self):
        """
        Gets and sets the lastModifiedDateTime
        
        Returns:
            datetime:
                The lastModifiedDateTime
        """
        if "lastModifiedDateTime" in self._prop_dict:
            if '.' in self._prop_dict["lastModifiedDateTime"]:
                return datetime.strptime(self._prop_dict["lastModifiedDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S.%f")
            else:
                return datetime.strptime(self._prop_dict["lastModifiedDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S")
        else:
            return None

    @last_modified_date_time.setter
    def last_modified_date_time(self, val):
        self._prop_dict["lastModifiedDateTime"] = val.isoformat()+((".0" if val.time().microsecond == 0 else "")+"Z")

    @property
    def name(self):
        """
        Gets and sets the name
        
        Returns:
            str:
                The name
        """
        if "name" in self._prop_dict:
            return self._prop_dict["name"]
        else:
            return None

    @name.setter
    def name(self, val):
        self._prop_dict["name"] = val

    @property
    def parent_reference(self):
        """
        Gets and sets the parentReference
        
        Returns: 
            :class:`ItemReference<onedrivesdk.model.item_reference.ItemReference>`:
                The parentReference
        """
        if "parentReference" in self._prop_dict:
            if isinstance(self._prop_dict["parentReference"], OneDriveObjectBase):
                return self._prop_dict["parentReference"]
            else :
                self._prop_dict["parentReference"] = ItemReference(self._prop_dict["parentReference"])
                return self._prop_dict["parentReference"]

        return None

    @parent_reference.setter
    def parent_reference(self, val):
        self._prop_dict["parentReference"] = val

    @property
    def size(self):
        """
        Gets and sets the size
        
        Returns:
            int:
                The size
        """
        if "size" in self._prop_dict:
            return self._prop_dict["size"]
        else:
            return None

    @size.setter
    def size(self, val):
        self._prop_dict["size"] = val

    @property
    def web_url(self):
        """
        Gets and sets the webUrl
        
        Returns:
            str:
                The webUrl
        """
        if "webUrl" in self._prop_dict:
            return self._prop_dict["webUrl"]
        else:
            return None

    @web_url.setter
    def web_url(self, val):
        self._prop_dict["webUrl"] = val

    @property
    def audio(self):
        """
        Gets and sets the audio
        
        Returns: 
            :class:`Audio<onedrivesdk.model.audio.Audio>`:
                The audio
        """
        if "audio" in self._prop_dict:
            if isinstance(self._prop_dict["audio"], OneDriveObjectBase):
                return self._prop_dict["audio"]
            else :
                self._prop_dict["audio"] = Audio(self._prop_dict["audio"])
                return self._prop_dict["audio"]

        return None

    @audio.setter
    def audio(self, val):
        self._prop_dict["audio"] = val

    @property
    def deleted(self):
        """
        Gets and sets the deleted
        
        Returns: 
            :class:`Deleted<onedrivesdk.model.deleted.Deleted>`:
                The deleted
        """
        if "deleted" in self._prop_dict:
            if isinstance(self._prop_dict["deleted"], OneDriveObjectBase):
                return self._prop_dict["deleted"]
            else :
                self._prop_dict["deleted"] = Deleted(self._prop_dict["deleted"])
                return self._prop_dict["deleted"]

        return None

    @deleted.setter
    def deleted(self, val):
        self._prop_dict["deleted"] = val

    @property
    def file(self):
        """
        Gets and sets the file
        
        Returns: 
            :class:`File<onedrivesdk.model.file.File>`:
                The file
        """
        if "file" in self._prop_dict:
            if isinstance(self._prop_dict["file"], OneDriveObjectBase):
                return self._prop_dict["file"]
            else :
                self._prop_dict["file"] = File(self._prop_dict["file"])
                return self._prop_dict["file"]

        return None

    @file.setter
    def file(self, val):
        self._prop_dict["file"] = val

    @property
    def file_system_info(self):
        """
        Gets and sets the fileSystemInfo
        
        Returns: 
            :class:`FileSystemInfo<onedrivesdk.model.file_system_info.FileSystemInfo>`:
                The fileSystemInfo
        """
        if "fileSystemInfo" in self._prop_dict:
            if isinstance(self._prop_dict["fileSystemInfo"], OneDriveObjectBase):
                return self._prop_dict["fileSystemInfo"]
            else :
                self._prop_dict["fileSystemInfo"] = FileSystemInfo(self._prop_dict["fileSystemInfo"])
                return self._prop_dict["fileSystemInfo"]

        return None

    @file_system_info.setter
    def file_system_info(self, val):
        self._prop_dict["fileSystemInfo"] = val

    @property
    def folder(self):
        """
        Gets and sets the folder
        
        Returns: 
            :class:`Folder<onedrivesdk.model.folder.Folder>`:
                The folder
        """
        if "folder" in self._prop_dict:
            if isinstance(self._prop_dict["folder"], OneDriveObjectBase):
                return self._prop_dict["folder"]
            else :
                self._prop_dict["folder"] = Folder(self._prop_dict["folder"])
                return self._prop_dict["folder"]

        return None

    @folder.setter
    def folder(self, val):
        self._prop_dict["folder"] = val

    @property
    def image(self):
        """
        Gets and sets the image
        
        Returns: 
            :class:`Image<onedrivesdk.model.image.Image>`:
                The image
        """
        if "image" in self._prop_dict:
            if isinstance(self._prop_dict["image"], OneDriveObjectBase):
                return self._prop_dict["image"]
            else :
                self._prop_dict["image"] = Image(self._prop_dict["image"])
                return self._prop_dict["image"]

        return None

    @image.setter
    def image(self, val):
        self._prop_dict["image"] = val

    @property
    def location(self):
        """
        Gets and sets the location
        
        Returns: 
            :class:`Location<onedrivesdk.model.location.Location>`:
                The location
        """
        if "location" in self._prop_dict:
            if isinstance(self._prop_dict["location"], OneDriveObjectBase):
                return self._prop_dict["location"]
            else :
                self._prop_dict["location"] = Location(self._prop_dict["location"])
                return self._prop_dict["location"]

        return None

    @location.setter
    def location(self, val):
        self._prop_dict["location"] = val

    @property
    def open_with(self):
        """
        Gets and sets the openWith
        
        Returns: 
            :class:`OpenWithSet<onedrivesdk.model.open_with_set.OpenWithSet>`:
                The openWith
        """
        if "openWith" in self._prop_dict:
            if isinstance(self._prop_dict["openWith"], OneDriveObjectBase):
                return self._prop_dict["openWith"]
            else :
                self._prop_dict["openWith"] = OpenWithSet(self._prop_dict["openWith"])
                return self._prop_dict["openWith"]

        return None

    @open_with.setter
    def open_with(self, val):
        self._prop_dict["openWith"] = val

    @property
    def photo(self):
        """
        Gets and sets the photo
        
        Returns: 
            :class:`Photo<onedrivesdk.model.photo.Photo>`:
                The photo
        """
        if "photo" in self._prop_dict:
            if isinstance(self._prop_dict["photo"], OneDriveObjectBase):
                return self._prop_dict["photo"]
            else :
                self._prop_dict["photo"] = Photo(self._prop_dict["photo"])
                return self._prop_dict["photo"]

        return None

    @photo.setter
    def photo(self, val):
        self._prop_dict["photo"] = val

    @property
    def remote_item(self):
        """
        Gets and sets the remoteItem
        
        Returns: 
            :class:`Item<onedrivesdk.model.item.Item>`:
                The remoteItem
        """
        if "remoteItem" in self._prop_dict:
            if isinstance(self._prop_dict["remoteItem"], OneDriveObjectBase):
                return self._prop_dict["remoteItem"]
            else :
                self._prop_dict["remoteItem"] = Item(self._prop_dict["remoteItem"])
                return self._prop_dict["remoteItem"]

        return None

    @remote_item.setter
    def remote_item(self, val):
        self._prop_dict["remoteItem"] = val

    @property
    def search_result(self):
        """
        Gets and sets the searchResult
        
        Returns: 
            :class:`SearchResult<onedrivesdk.model.search_result.SearchResult>`:
                The searchResult
        """
        if "searchResult" in self._prop_dict:
            if isinstance(self._prop_dict["searchResult"], OneDriveObjectBase):
                return self._prop_dict["searchResult"]
            else :
                self._prop_dict["searchResult"] = SearchResult(self._prop_dict["searchResult"])
                return self._prop_dict["searchResult"]

        return None

    @search_result.setter
    def search_result(self, val):
        self._prop_dict["searchResult"] = val

    @property
    def shared(self):
        """
        Gets and sets the shared
        
        Returns: 
            :class:`Shared<onedrivesdk.model.shared.Shared>`:
                The shared
        """
        if "shared" in self._prop_dict:
            if isinstance(self._prop_dict["shared"], OneDriveObjectBase):
                return self._prop_dict["shared"]
            else :
                self._prop_dict["shared"] = Shared(self._prop_dict["shared"])
                return self._prop_dict["shared"]

        return None

    @shared.setter
    def shared(self, val):
        self._prop_dict["shared"] = val

    @property
    def special_folder(self):
        """
        Gets and sets the specialFolder
        
        Returns: 
            :class:`SpecialFolder<onedrivesdk.model.special_folder.SpecialFolder>`:
                The specialFolder
        """
        if "specialFolder" in self._prop_dict:
            if isinstance(self._prop_dict["specialFolder"], OneDriveObjectBase):
                return self._prop_dict["specialFolder"]
            else :
                self._prop_dict["specialFolder"] = SpecialFolder(self._prop_dict["specialFolder"])
                return self._prop_dict["specialFolder"]

        return None

    @special_folder.setter
    def special_folder(self, val):
        self._prop_dict["specialFolder"] = val

    @property
    def video(self):
        """
        Gets and sets the video
        
        Returns: 
            :class:`Video<onedrivesdk.model.video.Video>`:
                The video
        """
        if "video" in self._prop_dict:
            if isinstance(self._prop_dict["video"], OneDriveObjectBase):
                return self._prop_dict["video"]
            else :
                self._prop_dict["video"] = Video(self._prop_dict["video"])
                return self._prop_dict["video"]

        return None

    @video.setter
    def video(self, val):
        self._prop_dict["video"] = val

    @property
    def permissions(self):
        """Gets and sets the permissions
        
        Returns: 
            :class:`PermissionsCollectionPage<onedrivesdk.request.permissions_collection.PermissionsCollectionPage>`:
                The permissions
        """
        if "permissions" in self._prop_dict:
            return PermissionsCollectionPage(self._prop_dict["permissions"])
        else:
            return None

    @property
    def subscriptions(self):
        """Gets and sets the subscriptions
        
        Returns: 
            :class:`SubscriptionsCollectionPage<onedrivesdk.request.subscriptions_collection.SubscriptionsCollectionPage>`:
                The subscriptions
        """
        if "subscriptions" in self._prop_dict:
            return SubscriptionsCollectionPage(self._prop_dict["subscriptions"])
        else:
            return None

    @property
    def versions(self):
        """Gets and sets the versions
        
        Returns: 
            :class:`VersionsCollectionPage<onedrivesdk.request.versions_collection.VersionsCollectionPage>`:
                The versions
        """
        if "versions" in self._prop_dict:
            return VersionsCollectionPage(self._prop_dict["versions"])
        else:
            return None

    @property
    def children(self):
        """Gets and sets the children
        
        Returns: 
            :class:`ChildrenCollectionPage<onedrivesdk.request.children_collection.ChildrenCollectionPage>`:
                The children
        """
        if "children" in self._prop_dict:
            return ChildrenCollectionPage(self._prop_dict["children"])
        else:
            return None

    @property
    def tags(self):
        """Gets and sets the tags
        
        Returns: 
            :class:`TagsCollectionPage<onedrivesdk.request.tags_collection.TagsCollectionPage>`:
                The tags
        """
        if "tags" in self._prop_dict:
            return TagsCollectionPage(self._prop_dict["tags"])
        else:
            return None

    @property
    def thumbnails(self):
        """Gets and sets the thumbnails
        
        Returns: 
            :class:`ThumbnailsCollectionPage<onedrivesdk.request.thumbnails_collection.ThumbnailsCollectionPage>`:
                The thumbnails
        """
        if "thumbnails" in self._prop_dict:
            return ThumbnailsCollectionPage(self._prop_dict["thumbnails"])
        else:
            return None

from ..model.permissions_collection_page import PermissionsCollectionPage
from ..model.subscriptions_collection_page import SubscriptionsCollectionPage
from ..model.versions_collection_page import VersionsCollectionPage
from ..model.children_collection_page import ChildrenCollectionPage
from ..model.tags_collection_page import TagsCollectionPage
from ..model.thumbnails_collection_page import ThumbnailsCollectionPage



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from ..model.item import Item
from ..collection_base import CollectionRequestBase
from ..request_builder_base import RequestBuilderBase
from ..options import *
import json
import asyncio


class ItemSearchRequest(CollectionRequestBase):

    def __init__(self, request_url, client, options, q=None):
        super(ItemSearchRequest, self).__init__(request_url, client, options)
        self.method = "GET"

        if q:
            self._query_options["q"] = q

    def get(self):
        """Sends the GET request
        
        Returns:
            :class:`ItemsCollectionResponse<onedrivesdk.request.items_collection.ItemsCollectionResponse>`:
                The resulting collection page from the operation
        """
        collection_response = ItemsCollectionResponse(json.loads(self.send().content))
        return self._page_from_response(collection_response)

    @asyncio.coroutine
    def get_async(self):
        """Sends the GET request using an asyncio coroutine
        
        Yields:
            :class:`ItemsCollectionResponse<onedrivesdk.request.items_collection.ItemsCollectionResponse>`:
                The resulting collection page from the operation
        """
        future = self._client._loop.run_in_executor(None,
                                                    self.get)
        collection_response = yield from future
        return collection_response
    
    @staticmethod
    def get_next_page_request(collection_page, client, options, q=None):
        """Gets the ItemSearchRequest for the next page. Returns None if there is no next page

        Yields: 
            :class:`ItemSearchRequest<onedrivesdk.request.item_search.ItemSearchRequest>`:
                The ItemSearchRequest
        """
        if collection_page._next_page_link:
            return ItemSearchRequest(collection_page._next_page_link, client, options, token)
        else:
            return None


class ItemSearchRequestBuilder(RequestBuilderBase):

    def __init__(self, request_url, client, q=None):
        super(ItemSearchRequestBuilder, self).__init__(request_url, client)
        self._method_options = {}

        self._method_options["q"] = q

    def request(self, expand=None, select=None, top=None, order_by=None, options=None):
        """Builds the request for the ItemSearch
        
        Args:
            expand (str): Default None, comma-seperated list of relationships
                to expand in the response.
            select (str): Default None, comma-seperated list of properties to
                include in the response.
            top (int): Default None, the number of items to return in a result.
            order_by (str): Default None, comma-seperated list of properties
                that are used to sort the order of items in the response.
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                Default to None, list of options to include in the request

        Returns: 
            :class:`ItemSearchRequest<onedrivesdk.request.item_search.ItemSearchRequest>`:
                The request
        """
        req = ItemSearchRequest(self._request_url, self._client, options, q=self._method_options["q"])
        req._set_query_options(expand=expand, select=select, top=top, order_by=order_by)
        return req

    def get(self):
        """Sends the GET request
        
        Returns:
            :class:`ItemsCollectionResponse<onedrivesdk.request.items_collection.ItemsCollectionResponse>`:
            The resulting ItemsCollectionResponse from the operation
        """
        return self.request().get()

    @asyncio.coroutine
    def get_async(self):
        """Sends the GET request using an asyncio coroutine
        
        Yields:
            :class:`ItemsCollectionResponse<onedrivesdk.request.items_collection.ItemsCollectionResponse>`:
                The resulting ItemsCollectionResponse from the operation
        """
        collection_page = yield from self.request().get_async()
        return collection_page

from ..request.items_collection import ItemsCollectionResponse



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.identity_set import IdentitySet
from ..one_drive_object_base import OneDriveObjectBase


class SharingInvitation(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def email(self):
        """Gets and sets the email
        
        Returns: 
            str:
                The email
        """
        if "email" in self._prop_dict:
            return self._prop_dict["email"]
        else:
            return None

    @email.setter
    def email(self, val):
        self._prop_dict["email"] = val

    @property
    def invited_by(self):
        """
        Gets and sets the invitedBy
        
        Returns: 
            :class:`IdentitySet<onedrivesdk.model.identity_set.IdentitySet>`:
                The invitedBy
        """
        if "invitedBy" in self._prop_dict:
            if isinstance(self._prop_dict["invitedBy"], OneDriveObjectBase):
                return self._prop_dict["invitedBy"]
            else :
                self._prop_dict["invitedBy"] = IdentitySet(self._prop_dict["invitedBy"])
                return self._prop_dict["invitedBy"]

        return None

    @invited_by.setter
    def invited_by(self, val):
        self._prop_dict["invitedBy"] = val
    @property
    def sign_in_required(self):
        """Gets and sets the signInRequired
        
        Returns: 
            bool:
                The signInRequired
        """
        if "signInRequired" in self._prop_dict:
            return self._prop_dict["signInRequired"]
        else:
            return None

    @sign_in_required.setter
    def sign_in_required(self, val):
        self._prop_dict["signInRequired"] = val

    @property
    def send_invitation_status(self):
        """Gets and sets the sendInvitationStatus
        
        Returns: 
            str:
                The sendInvitationStatus
        """
        if "sendInvitationStatus" in self._prop_dict:
            return self._prop_dict["sendInvitationStatus"]
        else:
            return None

    @send_invitation_status.setter
    def send_invitation_status(self, val):
        self._prop_dict["sendInvitationStatus"] = val

    @property
    def invite_error_resolve_url(self):
        """Gets and sets the inviteErrorResolveUrl
        
        Returns: 
            str:
                The inviteErrorResolveUrl
        """
        if "inviteErrorResolveUrl" in self._prop_dict:
            return self._prop_dict["inviteErrorResolveUrl"]
        else:
            return None

    @invite_error_resolve_url.setter
    def invite_error_resolve_url(self, val):
        self._prop_dict["inviteErrorResolveUrl"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from ..request_base import RequestBase
from ..model.subscription import Subscription
import json

class SubscriptionRequest(RequestBase):
    """The type SubscriptionRequest."""
    
    def __init__(self, request_url, client, options):
        """Constructs a new SubscriptionRequest.

        Args:
            request_url (str): The url to perform the SubscriptionRequest
                on
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request
        """
        super(SubscriptionRequest, self).__init__(request_url, client, options)

    def delete(self):
        """Deletes the specified Subscription."""
        self.method = "DELETE"
        self.send()

    def get(self):
        """Gets the specified Subscription.
        
        Returns:
            :class:`Subscription<onedrivesdk.model.subscription.Subscription>`:
                The Subscription.
        """
        self.method = "GET"
        entity = Subscription(json.loads(self.send().content))
        self._initialize_collection_properties(entity)
        return entity

    def update(self, subscription):
        """Updates the specified Subscription.
        
        Args:
            subscription (:class:`Subscription<onedrivesdk.model.subscription.Subscription>`):
                The Subscription to update.

        Returns:
            :class:`Subscription<onedrivesdk.model.subscription.Subscription>`:
                The updated Subscription.
        """
        self.content_type = "application/json"
        self.method = "PATCH"
        entity = Subscription(json.loads(self.send(subscription).content))
        self._initialize_collection_properties(entity)
        return entity




# -*- coding: utf-8 -*-
'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''

from ..error import OneDriveError
from ..model.upload_session import UploadSession
from ..model.item import Item
from ..options import HeaderOption
from ..request.item_request_builder import ItemRequestBuilder
from ..request_builder_base import RequestBuilderBase
from ..request_base import RequestBase
from ..helpers.file_slice import FileSlice
import asyncio
import json
import math
import os
import time

__PART_SIZE = 10 * 1024 * 1024  # recommended file size. Should be multiple of 320 * 1024
__MAX_SINGLE_FILE_UPLOAD = 100 * 1024 * 1024


class ItemUploadFragment(RequestBase):
    def __init__(self, request_url, client, options, file_handle):
        super(ItemUploadFragment, self).__init__(request_url, client, options)
        self.method = "PUT"
        self._file_handle = file_handle

    def post(self):
        """Sends the POST request

        Returns:
            :class:`UploadSession<onedrivesdk.model.upload_session.UploadSession>`:
                The resulting entity from the operation
        """
        entity = UploadSession(json.loads(self.send(data=self._file_handle).content))
        return entity

    @asyncio.coroutine
    def post_async(self):
        """Sends the POST request using an asyncio coroutine

        Yields:
            :class:`UploadedSession<onedrivesdk.model.upload_session.UploadedSession>`:
                The resulting entity from the operation
        """
        future = self._client._loop.run_in_executor(None,
                                                    self.post)
        entity = yield from future
        return entity


class ItemUploadFragmentBuilder(RequestBuilderBase):
    def __init__(self, request_url, client, content_local_path):
        super(ItemUploadFragmentBuilder, self).__init__(request_url, client)
        self._method_options = {}
        self._file_handle = open(content_local_path, "rb")
        self._total_length = os.stat(content_local_path).st_size

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self._file_handle.close()

    def request(self, begin, length, options=None):
        """Builds the request for the ItemUploadFragment

        Args:
            begin (int): First byte in range to be uploaded
            length (int): Number of bytes in range to be uploaded
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                Default to None, list of options to include in the request

        Returns:
            :class:`ItemUploadFragment<onedrivesdk.request.item_upload_fragment.ItemUploadFragment>`:
                The request
        """
        if not (options is None or len(options) == 0):
            opts = options.copy()
        else:
            opts = []

        self.content_type = "application/octet-stream"

        opts.append(HeaderOption("Content-Range", "bytes %d-%d/%d" % (begin, begin + length - 1, self._total_length)))
        opts.append(HeaderOption("Content-Length", str(length)))

        file_slice = FileSlice(self._file_handle, begin, length=length)
        req = ItemUploadFragment(self._request_url, self._client, opts, file_slice)
        return req

    def post(self, begin, length, options=None):
        """Sends the POST request

        Returns:
            :class:`UploadedFragment<onedrivesdk.model.uploaded_fragment.UploadedFragment>`:
            The resulting UploadSession from the operation
        """
        return self.request(begin, length, options).post()

    @asyncio.coroutine
    def post_async(self, begin, length, options=None):
        """Sends the POST request using an asyncio coroutine

        Yields:
            :class:`UploadedFragment<onedrivesdk.model.uploaded_fragment.UploadedFragment>`:
                The resulting UploadSession from the operation
        """
        entity = yield from self.request(begin, length, options).post_async()
        return entity


def fragment_upload_async(self, local_path, conflict_behavior=None, upload_status=None):
    """Uploads file using PUT using multipart upload if needed.

    Args:
        local_path (str): The path to the local file to upload.
        conflict_behavior (str): conflict behavior if the file is already
            uploaded. Use None value if file should be replaced or "rename", if
            the new file should get a new name
        upload_status (func): function(current_part, total_parts) to be called
            with upload status for each 10MB part

    Returns:
        Created entity.
    """
    file_size = os.stat(local_path).st_size
    if file_size <= __MAX_SINGLE_FILE_UPLOAD:
        # fallback to single shot upload if file is small enough
        return self.content.request().upload(local_path)
    else:
        # multipart upload needed for larger files
        if conflict_behavior:
            item = Item({'@name.conflictBehavior': conflict_behavior})
        else:
            item = Item({})

        session = self.create_session(item).post()

        with ItemUploadFragmentBuilder(session.upload_url, self._client, local_path) as upload_builder:
            total_parts = math.ceil(file_size / __PART_SIZE)
            for i in range(total_parts):
                if upload_status:
                    upload_status(i, total_parts)

                length = min(__PART_SIZE, file_size - i * __PART_SIZE)
                tries = 0
                while True:
                    try:
                        tries += 1
                        resp = upload_builder.post(i * __PART_SIZE, length)
                    except OneDriveError as exc:
                        if exc.status_code in (408, 500, 502, 503, 504) and tries < 5:
                            time.sleep(5)
                            continue
                        elif exc.status_code == 416:
                            # Fragment already received
                            break
                        elif exc.status_code == 401:
                            self._client.auth_provider.refresh_token()
                            continue
                        else:
                            raise exc
                    except ValueError:
                        # Swallow value errors (usually JSON error) and try again.
                        continue
                    break  # while True
        if upload_status:
            upload_status(total_parts, total_parts)  # job completed
        # return last response
        return resp

# Overwrite the standard upload operation to use this one
ItemRequestBuilder.upload_async = fragment_upload_async



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.open_with_app import OpenWithApp
from ..one_drive_object_base import OneDriveObjectBase


class OpenWithSet(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def web(self):
        """
        Gets and sets the web
        
        Returns: 
            :class:`OpenWithApp<onedrivesdk.model.open_with_app.OpenWithApp>`:
                The web
        """
        if "web" in self._prop_dict:
            if isinstance(self._prop_dict["web"], OneDriveObjectBase):
                return self._prop_dict["web"]
            else :
                self._prop_dict["web"] = OpenWithApp(self._prop_dict["web"])
                return self._prop_dict["web"]

        return None

    @web.setter
    def web(self, val):
        self._prop_dict["web"] = val
    @property
    def web_embed(self):
        """
        Gets and sets the webEmbed
        
        Returns: 
            :class:`OpenWithApp<onedrivesdk.model.open_with_app.OpenWithApp>`:
                The webEmbed
        """
        if "webEmbed" in self._prop_dict:
            if isinstance(self._prop_dict["webEmbed"], OneDriveObjectBase):
                return self._prop_dict["webEmbed"]
            else :
                self._prop_dict["webEmbed"] = OpenWithApp(self._prop_dict["webEmbed"])
                return self._prop_dict["webEmbed"]

        return None

    @web_embed.setter
    def web_embed(self, val):
        self._prop_dict["webEmbed"] = val



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.thumbnail import Thumbnail
from ..one_drive_object_base import OneDriveObjectBase


class ThumbnailSet(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def id(self):
        """
        Gets and sets the id
        
        Returns:
            str:
                The id
        """
        if "id" in self._prop_dict:
            return self._prop_dict["id"]
        else:
            return None

    @id.setter
    def id(self, val):
        self._prop_dict["id"] = val

    @property
    def large(self):
        """
        Gets and sets the large
        
        Returns: 
            :class:`Thumbnail<onedrivesdk.model.thumbnail.Thumbnail>`:
                The large
        """
        if "large" in self._prop_dict:
            if isinstance(self._prop_dict["large"], OneDriveObjectBase):
                return self._prop_dict["large"]
            else :
                self._prop_dict["large"] = Thumbnail(self._prop_dict["large"])
                return self._prop_dict["large"]

        return None

    @large.setter
    def large(self, val):
        self._prop_dict["large"] = val

    @property
    def medium(self):
        """
        Gets and sets the medium
        
        Returns: 
            :class:`Thumbnail<onedrivesdk.model.thumbnail.Thumbnail>`:
                The medium
        """
        if "medium" in self._prop_dict:
            if isinstance(self._prop_dict["medium"], OneDriveObjectBase):
                return self._prop_dict["medium"]
            else :
                self._prop_dict["medium"] = Thumbnail(self._prop_dict["medium"])
                return self._prop_dict["medium"]

        return None

    @medium.setter
    def medium(self, val):
        self._prop_dict["medium"] = val

    @property
    def small(self):
        """
        Gets and sets the small
        
        Returns: 
            :class:`Thumbnail<onedrivesdk.model.thumbnail.Thumbnail>`:
                The small
        """
        if "small" in self._prop_dict:
            if isinstance(self._prop_dict["small"], OneDriveObjectBase):
                return self._prop_dict["small"]
            else :
                self._prop_dict["small"] = Thumbnail(self._prop_dict["small"])
                return self._prop_dict["small"]

        return None

    @small.setter
    def small(self, val):
        self._prop_dict["small"] = val

    @property
    def source(self):
        """
        Gets and sets the source
        
        Returns: 
            :class:`Thumbnail<onedrivesdk.model.thumbnail.Thumbnail>`:
                The source
        """
        if "source" in self._prop_dict:
            if isinstance(self._prop_dict["source"], OneDriveObjectBase):
                return self._prop_dict["source"]
            else :
                self._prop_dict["source"] = Thumbnail(self._prop_dict["source"])
                return self._prop_dict["source"]

        return None

    @source.setter
    def source(self, val):
        self._prop_dict["source"] = val




from ..http_provider import HttpProvider
from ..auth_provider import AuthProvider
from ..request.one_drive_client import OneDriveClient


def get_default_client(client_id, scopes):
    """Deprecated. Proxy of :method:`get_consumer_client()`.
    Get a client using the default HttpProvider and
    AuthProvider classes.

    Args:
        client_id (str): The client id for your application
        scopes (list of str): The scopes required for your
            application

    Returns:
        :class:`OneDriveClient<onedrivesdk.requests.one_drive_client.OneDriveClient>`:
            A OneDriveClient for making OneDrive requests.
    """
    return get_consumer_client(client_id, scopes)

def get_consumer_client(client_id, scopes):
    """Get a client using the default HttpProvider and
    AuthProvider classes

    Args:
        client_id (str): The client id for your application
        scopes (list of str): The scopes required for your
            application

    Returns:
        :class:`OneDriveClient<onedrivesdk.requests.one_drive_client.OneDriveClient>`:
            A OneDriveClient for making OneDrive requests.
    """
    http_provider = HttpProvider()
    auth_provider = AuthProvider(http_provider=http_provider,
                                 client_id=client_id,
                                 scopes=scopes)
    return OneDriveClient("https://api.onedrive.com/v1.0/",
                          auth_provider,
                          http_provider)


def get_business_client(client_id, scopes, base_url):
    """Get a client using the default HttpProvider and
    AuthProvider classes

    Args:
        client_id (str): The client id for your application
        scopes (list of str): The scopes required for your
            application
        base_url (str): Base URL of OneDrive for Business tenant.
            For example, "https://my-sharepoint.contoso.com/v1.0/"

    Returns:
        :class:`OneDriveClient<onedrivesdk.requests.one_drive_client.OneDriveClient>`:
            A OneDriveClient for making OneDrive requests.
    """
    http_provider = HttpProvider()
    auth_provider = AuthProvider(http_provider=http_provider,
                                 client_id=client_id,
                                 scopes=scopes)
    return OneDriveClient(base_url,
                          auth_provider,
                          http_provider)




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from datetime import datetime
from ..one_drive_object_base import OneDriveObjectBase


class Photo(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def camera_make(self):
        """Gets and sets the cameraMake
        
        Returns: 
            str:
                The cameraMake
        """
        if "cameraMake" in self._prop_dict:
            return self._prop_dict["cameraMake"]
        else:
            return None

    @camera_make.setter
    def camera_make(self, val):
        self._prop_dict["cameraMake"] = val

    @property
    def camera_model(self):
        """Gets and sets the cameraModel
        
        Returns: 
            str:
                The cameraModel
        """
        if "cameraModel" in self._prop_dict:
            return self._prop_dict["cameraModel"]
        else:
            return None

    @camera_model.setter
    def camera_model(self, val):
        self._prop_dict["cameraModel"] = val

    @property
    def exposure_denominator(self):
        """Gets and sets the exposureDenominator
        
        Returns: 
            float:
                The exposureDenominator
        """
        if "exposureDenominator" in self._prop_dict:
            return self._prop_dict["exposureDenominator"]
        else:
            return None

    @exposure_denominator.setter
    def exposure_denominator(self, val):
        self._prop_dict["exposureDenominator"] = val

    @property
    def exposure_numerator(self):
        """Gets and sets the exposureNumerator
        
        Returns: 
            float:
                The exposureNumerator
        """
        if "exposureNumerator" in self._prop_dict:
            return self._prop_dict["exposureNumerator"]
        else:
            return None

    @exposure_numerator.setter
    def exposure_numerator(self, val):
        self._prop_dict["exposureNumerator"] = val

    @property
    def focal_length(self):
        """Gets and sets the focalLength
        
        Returns: 
            float:
                The focalLength
        """
        if "focalLength" in self._prop_dict:
            return self._prop_dict["focalLength"]
        else:
            return None

    @focal_length.setter
    def focal_length(self, val):
        self._prop_dict["focalLength"] = val

    @property
    def f_number(self):
        """Gets and sets the fNumber
        
        Returns: 
            float:
                The fNumber
        """
        if "fNumber" in self._prop_dict:
            return self._prop_dict["fNumber"]
        else:
            return None

    @f_number.setter
    def f_number(self, val):
        self._prop_dict["fNumber"] = val

    @property
    def taken_date_time(self):
        """Gets and sets the takenDateTime
        
        Returns: 
            datetime:
                The takenDateTime
        """
        if "takenDateTime" in self._prop_dict:
            if '.' in self._prop_dict["takenDateTime"]:
                return datetime.strptime(self._prop_dict["takenDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S.%f")
            else:
                return datetime.strptime(self._prop_dict["takenDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S")
        else:
            return None

    @taken_date_time.setter
    def taken_date_time(self, val):
        self._prop_dict["takenDateTime"] = val.isoformat()+((".0" if val.time().microsecond == 0 else "")+"Z")

    @property
    def iso(self):
        """Gets and sets the iso
        
        Returns: 
            int:
                The iso
        """
        if "iso" in self._prop_dict:
            return self._prop_dict["iso"]
        else:
            return None

    @iso.setter
    def iso(self, val):
        self._prop_dict["iso"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from datetime import datetime
from ..one_drive_object_base import OneDriveObjectBase


class UploadSession(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def upload_url(self):
        """Gets and sets the uploadUrl
        
        Returns: 
            str:
                The uploadUrl
        """
        if "uploadUrl" in self._prop_dict:
            return self._prop_dict["uploadUrl"]
        else:
            return None

    @upload_url.setter
    def upload_url(self, val):
        self._prop_dict["uploadUrl"] = val

    @property
    def expiration_date_time(self):
        """Gets and sets the expirationDateTime
        
        Returns: 
            datetime:
                The expirationDateTime
        """
        if "expirationDateTime" in self._prop_dict:
            if '.' in self._prop_dict["expirationDateTime"]:
                return datetime.strptime(self._prop_dict["expirationDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S.%f")
            else:
                return datetime.strptime(self._prop_dict["expirationDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S")
        else:
            return None

    @expiration_date_time.setter
    def expiration_date_time(self, val):
        self._prop_dict["expirationDateTime"] = val.isoformat()+((".0" if val.time().microsecond == 0 else "")+"Z")

    @property
    def next_expected_ranges(self):
        """Gets and sets the nextExpectedRanges
        
        Returns: 
            str:
                The nextExpectedRanges
        """
        if "nextExpectedRanges" in self._prop_dict:
            return self._prop_dict["nextExpectedRanges"]
        else:
            return None

    @next_expected_ranges.setter
    def next_expected_ranges(self, val):
        self._prop_dict["nextExpectedRanges"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from ..request_base import RequestBase
from ..model.thumbnail_set import ThumbnailSet
import json
import asyncio

class ThumbnailSetRequest(RequestBase):
    """The type ThumbnailSetRequest."""
    
    def __init__(self, request_url, client, options):
        """Constructs a new ThumbnailSetRequest.

        Args:
            request_url (str): The url to perform the ThumbnailSetRequest
                on
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request
        """
        super(ThumbnailSetRequest, self).__init__(request_url, client, options)

    def delete(self):
        """Deletes the specified ThumbnailSet."""
        self.method = "DELETE"
        self.send()

    @asyncio.coroutine
    def delete_async(self):
        """Deletes the specified ThumbnailSet."""
        future = self._client._loop.run_in_executor(None,
                                                    self.delete)
        yield from future

    def get(self):
        """Gets the specified ThumbnailSet.
        
        Returns:
            :class:`ThumbnailSet<onedrivesdk.model.thumbnail_set.ThumbnailSet>`:
                The ThumbnailSet.
        """
        self.method = "GET"
        entity = ThumbnailSet(json.loads(self.send().content))
        self._initialize_collection_properties(entity)
        return entity

    @asyncio.coroutine
    def get_async(self):
        """Gets the specified ThumbnailSet in async.

        Yields:
            :class:`ThumbnailSet<onedrivesdk.model.thumbnail_set.ThumbnailSet>`:
                The ThumbnailSet.
        """
        future = self._client._loop.run_in_executor(None,
                                                    self.get)
        entity = yield from future
        return entity

    def update(self, thumbnail_set):
        """Updates the specified ThumbnailSet.
        
        Args:
            thumbnail_set (:class:`ThumbnailSet<onedrivesdk.model.thumbnail_set.ThumbnailSet>`):
                The ThumbnailSet to update.

        Returns:
            :class:`ThumbnailSet<onedrivesdk.model.thumbnail_set.ThumbnailSet>`:
                The updated ThumbnailSet.
        """
        self.content_type = "application/json"
        self.method = "PATCH"
        entity = ThumbnailSet(json.loads(self.send(thumbnail_set).content))
        self._initialize_collection_properties(entity)
        return entity

    @asyncio.coroutine
    def update_async(self, thumbnail_set):
        """Updates the specified ThumbnailSet in async
        
        Args:
            thumbnail_set (:class:`ThumbnailSet<onedrivesdk.model.thumbnail_set.ThumbnailSet>`):
                The ThumbnailSet to update.

        Yields:
            :class:`ThumbnailSet<onedrivesdk.model.thumbnail_set.ThumbnailSet>`:
                The updated ThumbnailSet.
        """
        future = self._client._loop.run_in_executor(None,
                                                    self.update,
                                                    thumbnail_set)
        entity = yield from future
        return entity

